import pytest
from rest_framework import status


class TestTranslatePagination:
    """Test pagination."""

    @pytest.mark.django_db
    @pytest.mark.xdist_group(name="german_translations")
    def test_paginated_response(self, make_request, import_german_translations_fixture):
        from translate.service.views import TranslationsAPIView

        kwargs = {"language": "de"}
        data = {"page": 1, "page_size": 5}
        request = make_request(f"get::api_translations", kwargs=kwargs, data=data)
        response = TranslationsAPIView.as_view()(request, **kwargs)

        assert response.status_code == status.HTTP_200_OK
        assert response.data, "Response should return some data"
        assert {"links", "pages", "count", "results"} == set(response.data.keys())
        assert isinstance(response.data.get("results"), list)
        assert not len(response.data.get("results")) > 5, "Should not return more than page_size values"



import csv
from pathlib import Path

from django.core.management.base import BaseCommand
from django.db.models import OuterRef, Subquery

from translate.core.utils.logger import log
from translate.service.models import Translation


class Command(BaseCommand):
    """
    Export translations in the format:

    key,views,context,translation_de,translation_en,translation_fr,translation_it,translation_plural_de,
    translation_plural_en,translation_plural_fr,translation_plural_it
    """

    help = "Export database translations to CSV file in the bulk import format"

    @staticmethod
    def get_translation_subquery(language_code, plural=False):
        """Helper function to create a subquery for fetching translations."""
        if plural:
            return Subquery(
                Translation.objects.filter(key=OuterRef("key"), language__lang_info=language_code).values(
                    "translation_plural"
                )[:1]
            )
        else:
            return Subquery(
                Translation.objects.filter(key=OuterRef("key"), language__lang_info=language_code).values(
                    "translation"
                )[:1]
            )

    def export_csv(self):
        _DB_READ_PAGE_SIZE = 500
        _CSV_HEADER = [
            "key",
            "views",
            "context",
            "translation_de",
            "translation_en",
            "translation_fr",
            "translation_it",
            "translation_plural_de",
            "translation_plural_en",
            "translation_plural_fr",
            "translation_plural_it",
        ]

        file_name = Path("translations.csv")
        log.info(f"Exporting translations to {file_name.absolute()}")

        qs = (
            Translation.objects.exclude(key__snake_name__startswith="snake_name_")
            .order_by("key__snake_name")
            .values("key__snake_name", "key__usage_context", "key__views")
            .annotate(
                translation_de=self.get_translation_subquery("de"),
                translation_en=self.get_translation_subquery("en"),
                translation_fr=self.get_translation_subquery("fr"),
                translation_it=self.get_translation_subquery("it"),
                translation_plural_de=self.get_translation_subquery("de", plural=True),
                translation_plural_en=self.get_translation_subquery("en", plural=True),
                translation_plural_fr=self.get_translation_subquery("fr", plural=True),
                translation_plural_it=self.get_translation_subquery("it", plural=True),
            )
            .distinct("key__snake_name")
        )

        with file_name.open(mode="w") as csvfile:
            writer = csv.DictWriter(csvfile, fieldnames=_CSV_HEADER)
            writer.writeheader()

            for record in qs:
                writer.writerow(
                    {
                        "key": record["key__snake_name"],
                        "views": ":".join(record["key__views"]),
                        "context": record["key__usage_context"],
                        "translation_de": record["translation_de"] or "",
                        "translation_en": record["translation_en"] or "",
                        "translation_fr": record["translation_fr"] or "",
                        "translation_it": record["translation_it"] or "",
                        "translation_plural_de": record["translation_plural_de"] or "",
                        "translation_plural_en": record["translation_plural_en"] or "",
                        "translation_plural_fr": record["translation_plural_fr"] or "",
                        "translation_plural_it": record["translation_plural_it"] or "",
                    }
                )

        log.info("Export completed successfully!")

    def handle(self, *args, **options):
        """Handle command."""
        try:
            self.export_csv()
        except Exception as ex:
            log.error("An error occurred while exporting translations: %s", ex)

        log.info("Done!")



import csv
from pathlib import Path

from django.conf.locale import LANG_INFO
from django.core.management.base import BaseCommand

from translate.core.utils.logger import log
from translate.service.models import Language, Translation


class Command(BaseCommand):
    """Export translations in following format:  language, key, context, translation, translation_plural"""

    help = "Export database translations to CSV file"

    def add_arguments(self, parser):
        """Attach argument for import_translations command."""
        parser.add_argument(
            "languages",
            type=str,
            nargs="+",
            default=["de"],
        )

    @staticmethod
    def export_csv(language: Language):
        """Export translations to CSV file."""
        _DB_READ_PAGE_SIZE = 500
        _CSV_HEADER = ["language", "key", "context", "translation", "translation_plural"]

        file_name = Path(f"translations_{language.lang_info}.csv")
        log.info(f"Exporting translations to {file_name.absolute()}")

        qs = (
            Translation.objects.filter(language=language)
            .exclude(key__snake_name__startswith="snake_name_")
            .order_by("key__snake_name")
        )

        with file_name.open(mode="w") as csvfile:
            writer = csv.DictWriter(csvfile, fieldnames=_CSV_HEADER)

            writer.writeheader()
            for record in qs.iterator(chunk_size=_DB_READ_PAGE_SIZE):
                record = {
                    "language": record.language.lang_info,
                    "key": record.key.snake_name,
                    "context": record.key.usage_context,
                    "translation": record.translation,
                    "translation_plural": record.translation_plural,
                }
                log.info(f"Writing record {record}")
                writer.writerow(record)

    def handle(self, *args, **options):
        """Handle command."""
        languages = options.get("languages")
        for language in languages:
            if language not in LANG_INFO:
                raise RuntimeError(f"Language {language} not found.")

            try:
                language = Language.objects.get(lang_info=language)
                self.export_csv(language)
            except Language.DoesNotExist:
                log.error(f"Language {language} not found in database.")
                continue

        log.info("Done!")



from django.core import management


class Command(management.BaseCommand):
    def handle(self, *args, **options):
        management.call_command(
            "bulk_import_translations_from_csv", "translate/service/import_spreadsheets/translations.csv"
        )



from pathlib import Path

import sentry_sdk
from decouple import config
from sentry_sdk.integrations.django import DjangoIntegration

from translate.service.utils.version_handler import extract_version

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent.parent

VERSION_TAG = extract_version()

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "django-insecure-yq@*8hm7=0d1mi@lqnnmh$qalu0uv0@m1a0e!_1o09wt%++4^k"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = config("DEBUG", cast=bool, default=True)

ALLOWED_HOSTS = ["*"]

sentry_sdk.init(
    dsn=config("SENTRY_TRANSLATE_DNS", cast=str, default=""),
    environment=config("SENTRY_ENVIRONMENT", default="development", cast=str),
    integrations=[
        DjangoIntegration(),
    ],
    # Set traces_sample_rate to 1.0 to capture 100%
    # of transactions for performance monitoring.
    # We recommend adjusting this value in production.
    traces_sample_rate=1.0,
    # If you wish to associate users to errors (assuming you are using
    # django.contrib.auth) you may enable sending PII data.
    send_default_pii=True,
)

TRANSLATE_ADDRESS = config("TRANSLATE_ADDRESS", cast=str, default="http://localhost")
TRANSLATE_PORT = config("TRANSLATE_PORT", cast=int, default="8000")

# Application definition

INSTALLED_APPS = [
    "baton",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django_admin_inline_paginator",
    "auditlog",
    "rest_framework",
    "drf_spectacular",
    "translate.service",
    "health_check",
    "health_check.db",
    "health_check.cache",
    "health_check.storage",
    "health_check.contrib.migrations",
    "baton.autodiscover",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "whitenoise.middleware.WhiteNoiseMiddleware",
    "django.middleware.gzip.GZipMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "auditlog.middleware.AuditlogMiddleware",
]

ROOT_URLCONF = "translate.core.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            "translate/service/templates/",
        ],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

STORAGES = {
    "default": {
        "BACKEND": "django.core.files.storage.FileSystemStorage",
    },
    "staticfiles": {
        "BACKEND": "whitenoise.storage.CompressedManifestStaticFilesStorage",
    },
}

WSGI_APPLICATION = "translate.core.wsgi.application"


# Database
# https://docs.djangoproject.com/en/4.0/ref/settings/#databases
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": config("POSTGRES_DB", default="melon_translate"),
        "USER": config("POSTGRES_USER", default="melon_user"),
        "PASSWORD": config("POSTGRES_PASSWORD", default="melonmelon!"),
        "HOST": config("POSTGRES_HOST", default="127.0.0.1", cast=str),
        "PORT": config("POSTGRES_PORT", cast=int, default=5432),
        "CONN_MAX_AGE": config("POSTGRES_CONN_MAX_AGE", cast=int, default=600),
    },
}

REDIS_HOST = config("REDIS_HOST", cast=str, default="127.0.0.1")
REDIS_PORT = config("REDIS_PORT", cast=int, default=6379)
REDIS_DB = config("REDIS_DATABASE", cast=int, default=0)

CACHES = {
    "default": {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": f"redis://{REDIS_HOST}:{REDIS_PORT}/{REDIS_DB}",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        },
    }
}

CACHE_TTL = 60 * 90  # 90 minutes
SESSION_ENGINE = "django.contrib.sessions.backends.cache"
SESSION_CACHE_ALIAS = "default"

# CSRF settings
CSRF_TRUSTED_ORIGINS = config(
    "CSRF_TRUSTED_ORIGINS", cast=lambda v: [s.strip() for s in v.split(",")], default="http://localhost"
)

# Password validation
# https://docs.djangoproject.com/en/4.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.0/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.0/howto/static-files

STATIC_ROOT = (Path(BASE_DIR) / "assets").as_posix()

STATIC_URL = "/static/"


# Default primary key field type
# https://docs.djangoproject.com/en/4.0/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

REST_FRAMEWORK = {
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
    "DEFAULT_PAGINATION_CLASS": "translate.service.paginations.TranslateClientPagination",
    "PAGE_SIZE": 500,  # default value if not set in paginator
}

SPECTACULAR_SETTINGS = {
    "TITLE": "melon-translate",
    "DESCRIPTION": "Translation microservice",
    "VERSION": VERSION_TAG,
}

LOGGING = {
    "version": 1,
    "disable_existing_loggers": True,
    "handlers": {
        "json": {
            "class": "logging.StreamHandler",
            "formatter": "json",
        },
    },
    "formatters": {
        "json": {
            "()": "pythonjsonlogger.jsonlogger.JsonFormatter",
            "format": "%(asctime)s %(levelname)s %(name)s %(message)s",
        },
    },
    "root": {
        "handlers": ["json"],
        "level": "INFO",
    },
}

BATON = {
    "SITE_HEADER": "Melon-translate",
    "SITE_TITLE": "Melon-translate",
    "INDEX_TITLE": "Translations administration",
    "SUPPORT_HREF": "https://github.com/",
    "COPYRIGHT": 'Copyright © 2022 <a href="https://www.emonitor.ch">eMonitor</a>',  # noqa
    "POWERED_BY": '<a href="https://www.emonitor.ch">eMonitor</a>',
    "CONFIRM_UNSAVED_CHANGES": True,
    "SHOW_MULTIPART_UPLOADING": True,
    "ENABLE_IMAGES_PREVIEW": True,
    "CHANGELIST_FILTERS_IN_MODAL": True,
    "CHANGELIST_FILTERS_ALWAYS_OPEN": False,
    "CHANGELIST_FILTERS_FORM": True,
    "MENU_ALWAYS_COLLAPSED": False,
    "MENU_TITLE": "Menu",
    "MESSAGES_TOASTS": False,
    "GRAVATAR_DEFAULT_IMG": "retro",
    "LOGIN_SPLASH": f"{STATIC_URL}assets/img/swiss_flg.jpeg",
    "SEARCH_FIELD": {
        "label": "Search contents...",
        "url": "/search/",
    },
    "MENU": (
        {"type": "title", "label": "main", "apps": ("auth",)},
        {
            "type": "app",
            "name": "auth",
            "label": "Authentication",
            "icon": "fa fa-lock",
            "models": (
                {"name": "user", "label": "Users"},
                {"name": "group", "label": "Groups"},
            ),
        },
        {"type": "title", "label": "service"},
        {
            "type": "free",
            "name": "translate.service",
            "label": "Translation service",
            "icon": "fa fa-sign",
            "url": "/admin/service/",
        },
        {
            "type": "free",
            "label": "Add New Translation Key",
            "default_open": True,
            "icon": "fa fa-key",
            "url": "/admin/service/translationkey/add/",
        },
        {
            "type": "free",
            "name": "context",
            "label": "Usage context input",
            "icon": "fa fa-graduation-cap",
            "url": "/admin/service/translationkey/?context-input=context-not-inserted/",
        },
        {
            "type": "free",
            "name": "health",
            "label": "Health check",
            "icon": "fa fa-heart",
            "url": "/health/",
        },
        {
            "type": "free",
            "name": "redoc",
            "label": "Redoc schema docs",
            "icon": "fa fa-book",
            "url": "/api/schema/redoc/",
        },
    ),
}

SILENCED_SYSTEM_CHECKS = [
    "django_jsonfield_backport.W001",
]



from baton.admin import SimpleDropdownFilter
from django_admin_inline_paginator.admin import TabularInlinePaginated

from translate.service.models import Translation, TranslationKey

FILTER_TRANSLATIONS_MIMO = "mimo"
FILTER_TRANSLATIONS_MELON = "melon"
FILTER_KEYS_MIMO = "mimo-keys"
FILTER_KEYS_MELON = "melon-keys"
FILTER_CONTEXT = "context-inserted"
FILTER_NO_CONTEXT = "context-not-inserted"


class TranslationInline(TabularInlinePaginated):
    fields = ("translation", "translation_plural", "language")
    per_page = 7
    model = Translation


class IgnoreMimoTranslationsFilter(SimpleDropdownFilter):
    """
    This is a filter for filtering translations by heritage. Mimo translations are old
    translations, used in legacy codebase, and melon are new and currently used ones.
    """

    title = "Ignoring mimo translations"
    parameter_name = "mimo"

    def lookups(self, request, model_admin):
        return (
            (FILTER_TRANSLATIONS_MIMO, "Old mimo translations"),
            (FILTER_TRANSLATIONS_MELON, "New translations"),
        )

    def queryset(self, request, queryset):
        if self.value() == FILTER_TRANSLATIONS_MIMO:
            return Translation.objects.filter(key__snake_name__startswith="snake_name_").order_by("-modified")
        if self.value() == FILTER_TRANSLATIONS_MELON:
            return Translation.objects.exclude(key__snake_name__startswith="snake_name_").order_by("-modified")


class IgnoreMimoKeysFilter(SimpleDropdownFilter):
    """
    This is a filter for filtering keys by heritage. Mimo translation keys are old
    translations, used in legacy codebase, and melon are new and currently used ones.
    """

    title = "Ignoring mimo keys"
    parameter_name = "mimo-keys"

    def lookups(self, request, model_admin):
        return [
            (FILTER_KEYS_MIMO, "Old mimo translation keys"),
            (FILTER_KEYS_MELON, "New translation keys"),
        ]

    def queryset(self, request, queryset):
        if self.value() == FILTER_KEYS_MIMO:
            return TranslationKey.objects.filter(snake_name__startswith="snake_name_").order_by("-modified")
        if self.value() == FILTER_KEYS_MELON:
            return TranslationKey.objects.exclude(snake_name__startswith="snake_name_").order_by("-modified")


class ContextInputFilter(SimpleDropdownFilter):
    title = "context"
    parameter_name = "context-input"

    def lookups(self, request, model_admin):
        return [
            (FILTER_CONTEXT, "Inserted"),
            (FILTER_NO_CONTEXT, "Not inserted"),
        ]

    def queryset(self, request, queryset):
        if self.value() == FILTER_CONTEXT:
            return TranslationKey.objects.filter(usage_context__isnull=False).order_by("-modified")
        if self.value() == FILTER_NO_CONTEXT:
            return TranslationKey.objects.filter(usage_context__isnull=True).order_by("-modified")



from typing import Optional

from rest_framework import pagination
from rest_framework.response import Response


class TranslateClientPagination(pagination.PageNumberPagination):
    """TranslateClient Pagination class."""

    page_size = 3000
    page_size_query_param = "page_size"
    page_query_param = "page"

    def get_next_page(self) -> Optional[int]:
        if self.page.has_next():
            return self.page.next_page_number()
        return None

    def get_paginated_response(self, data):
        return Response(
            {
                "count": self.page.paginator.count,
                "links": {"next": self.get_next_link(), "previous": self.get_previous_link()},
                "pages": {"current": self.page.number, "next": self.get_next_page()},
                "results": list(data),
            }
        )



from django.contrib import admin
from django.db import models
from django.forms import TextInput

from translate.service.filters import (
    ContextInputFilter,
    IgnoreMimoKeysFilter,
    IgnoreMimoTranslationsFilter,
    TranslationInline,
)
from translate.service.models import Language, Translation, TranslationKey


@admin.register(Translation)
class TranslationAdmin(admin.ModelAdmin):
    """Admin registration for ``Translation``."""

    formfield_overrides = {models.TextField: {"widget": TextInput()}}
    search_fields = ("key__snake_name", "key__id_name", "translation")
    fields = ("translation", "translation_plural", "language")

    list_display = ("key", "translation", "translation_plural", "language")
    list_filter = (
        "language",
        ("translation", admin.EmptyFieldListFilter),
        "key__category",
        IgnoreMimoTranslationsFilter,
    )
    list_editable = ("translation", "translation_plural")
    list_per_page = 15

    save_on_top = True


@admin.register(Language)
class LanguageAdmin(admin.ModelAdmin):
    """Admin registration for ``Language``."""

    list_display = ("lang_info",)
    save_on_top = True


@admin.register(TranslationKey)
class TranslationKeyAdmin(admin.ModelAdmin):
    """Admin registration for ``TranslationKey.``"""

    formfield_overrides = {models.TextField: {"widget": TextInput()}}
    fields = ("snake_name", "id_name", "usage_context", "category", "views", "occurrences")
    inlines = (TranslationInline,)
    search_fields = ("snake_name", "id_name", "id")

    list_display = ("snake_name", "id_name", "usage_context", "category")
    list_filter = (
        "category",
        "created",
        "modified",
        IgnoreMimoKeysFilter,
        ContextInputFilter,
    )
    list_editable = ("usage_context", "category")
    list_per_page = 15

    save_on_top = True



import uuid

import pytest
from django.db.models import Q, RestrictedError

from translate.core.settings import BASE_DIR
from translate.service.utils.file_utils import read_json_files, read_po_files


@pytest.fixture
def readiness_provider(provider_to_fixture):
    """Readiness provider fixture."""
    from translate.service.tests.providers import ReadinessCheckProvider

    provider_to_fixture.add_provider(ReadinessCheckProvider)

    yield provider_to_fixture


@pytest.fixture(scope="session")
@pytest.mark.xdist_group(name="group_db")
def language_factory():
    """Language factory fixture."""
    from translate.service.tests.factories import LanguageFactory

    obj = LanguageFactory.create()

    yield obj

    obj.delete()


@pytest.fixture(
    params=[
        "general_accept",
        "1another_one",
        "$my_translation",
        "my msgid " "%$#$@@#534",
    ],
)
@pytest.mark.xdist_group(name="group_db")
def translation_key_factory(request, worker_id):
    """``TranslationKey`` factory fixture."""
    from translate.service.models import TranslationKey as Tk
    from translate.service.tests.factories import TranslationKeyFactory

    obj = TranslationKeyFactory.create(
        snake_name=f"{request.param}_{worker_id}",
        category=Tk.Category.SERVICE,
    )

    yield obj

    obj.delete()


@pytest.fixture(
    params=[
        {
            "keys": [
                {
                    "key": "$my_context_key",
                    "usage_context": "$my_translation context",
                },
                {
                    "key": "and_1another_one",
                    "usage_context": "1another_one context",
                },
            ]
        },
    ],
)
def translation_key_with_context_factory(request):
    """``TranslationKey`` that have usage context inserted factory fixture."""
    from translate.service.models import TranslationKey as Tk
    from translate.service.tests.factories import TranslationKeyFactory

    tks = [
        TranslationKeyFactory.create(
            snake_name=f"{tk.get('key')}_{uuid.uuid4().hex}",
            category=Tk.Category.SERVICE,
            usage_context=tk.get("usage_context"),
        )
        for tk in request.param.get("keys")
    ]

    yield tks

    for tk in tks:
        tk.delete()


@pytest.fixture(
    params=[
        "snake_name_one",
        "snake_name_two2",
        "snake_name_%$#@%^&@",
    ],
)
def translation_key_mimo_factory(request):
    """``TranslationKey`` with mimo snake_names factory fixture."""
    from translate.service.models import TranslationKey as Tk
    from translate.service.tests.factories import TranslationKeyFactory

    tk = TranslationKeyFactory.create(
        snake_name=f"{request.param}_{uuid.uuid4().hex}",
        category=Tk.Category.SERVICE,
    )

    yield tk
    tk.delete()


@pytest.fixture
@pytest.mark.xdist_group(name="group_db")
def translation_factory(language_factory, translation_key_factory):
    """Translation factory fixture."""
    from translate.service.tests.factories import TranslationFactory

    obj = TranslationFactory.create(language=language_factory, key=translation_key_factory)

    yield obj, language_factory, translation_key_factory

    obj.delete()


@pytest.fixture
@pytest.mark.xdist_group(name="group_db")
def translation_mimo_factory(language_factory, translation_key_mimo_factory):
    """Translation factory fixture."""
    from translate.service.tests.factories import TranslationFactory

    obj = TranslationFactory.create(language=language_factory, key=translation_key_mimo_factory)

    yield obj, language_factory, translation_key_mimo_factory
    obj.delete()


@pytest.fixture
def translation_provider(provider_to_fixture):
    """Translation provider fixture."""
    from translate.service.tests.providers import TranslationProvider

    provider_to_fixture.add_provider(TranslationProvider)

    yield provider_to_fixture


@pytest.fixture
def language_provider(provider_to_fixture):
    """Language provider fixture."""
    from translate.service.tests.providers import LanguageProvider

    provider_to_fixture.add_provider(LanguageProvider)

    yield provider_to_fixture


@pytest.fixture
def translation_key_provider(provider_to_fixture):
    """Translation key provider."""
    from translate.service.tests.providers import TranslationKeyProvider

    provider_to_fixture.add_provider(TranslationKeyProvider)

    yield provider_to_fixture


@pytest.fixture
def user_account(worker_id):
    """Use a different account in each xdist worker"""
    return f"account_{worker_id}"


@pytest.fixture
def import_german_translations_fixture():
    """Inserts data into test database before running tests. Needed for ``client`` tests."""
    from django.core.management import call_command

    from translate.service.models import Language, Translation, TranslationKey

    translations_used = [*read_po_files("german_fixtures")]

    used_json_translations = read_json_files("german_fixtures", "snake_translations")
    json_translations_snake_names = used_json_translations.get("de").get("translation_center_marketing")
    for key in json_translations_snake_names.keys():
        translations_used.append(json_translations_snake_names[key]["translations"])

    call_command("import_translations", translations_dir="translate/service/tests/fixtures/german_fixtures")
    yield

    Translation.objects.filter(
        translation__in=translations_used,
        language__lang_info="de",
    ).delete()
    TranslationKey.objects.filter(
        Q(snake_name__in=json_translations_snake_names.keys()) | Q(id_name__in=translations_used)
    ).delete()
    Language.objects.filter(lang_info="de").delete()


@pytest.fixture
def import_french_translations_fixture():
    """Inserts data into test database before running tests. Needed for ``client`` tests."""
    from django.core.management import call_command

    from translate.service.models import Language, Translation, TranslationKey

    Translation.objects.all().delete()
    TranslationKey.objects.all().delete()
    Language.objects.all().delete()

    call_command("import_translations", translations_dir="translate/service/tests/fixtures/french_fixtures")
    yield

    Translation.objects.all().delete()
    TranslationKey.objects.all().delete()
    Language.objects.all().delete()


@pytest.fixture
def request_serializer_fixture(make_request):
    from translate.service.serializers import TranslationRequestSerializer

    lang = {"language": "de"}
    arguments = {
        "views": ["translation_center_frontend", "translations_center_placeholders"],
        "page": 1,
        "page_size": 5,
    }
    data = dict(lang, **arguments)
    yield TranslationRequestSerializer(data=data)


@pytest.fixture
def export_csv_fixture(import_german_translations_fixture):
    """Yields path of generated CSV file"""
    import os
    from pathlib import Path

    from django.core.management import call_command

    language_code = "de"
    file_name = f"translations_{language_code}.csv"

    call_command("export_csv", language_code)
    file_path = Path(file_name)

    yield file_path

    os.remove(file_name)


@pytest.fixture()
def melon_languages_factory():
    """``Language`` factory fixture. Creates languages used in melon project and returns them as a list."""
    from translate.service.models import Language

    objs = []
    lang_list = ["en", "de", "fr"]
    for lang_code in lang_list:
        lang, _ = Language.objects.get_or_create(lang_info=lang_code)
        objs.append(lang)

    yield objs

    for obj in objs:
        try:
            obj.delete()
        except RestrictedError:
            continue


@pytest.fixture
def bulk_import_translations_from_csv_fixture(melon_languages_factory):
    """Yields path of generated CSV file"""
    from django.core.management import call_command

    file_name = f"{BASE_DIR}/translate/service/tests/fixtures/bulk_import_translations_from_csv_test_fixture.csv"
    call_command("bulk_import_translations_from_csv", file_name)
    yield



import pytest


class TestLanguageSerializer:
    """Tests for ``Language`` model serializer."""

    @pytest.mark.serializer
    @pytest.mark.django_db
    def test_serialization_expected_fields(self, language_factory):
        """Check expected fields after serializing ``Language`` model."""
        from translate.service.serializers import LanguageSerializer

        serializer = LanguageSerializer(language_factory)
        assert {"lang_info", "id"} == set(serializer.data)
        assert serializer.data.get("lang_info")
        assert serializer.data.get("id")

    @pytest.mark.serializer
    @pytest.mark.django_db
    def test_deserialization_expected_fields(self, language_provider):
        """Check model instance creation and validation through serializer."""
        from translate.service.serializers import LanguageSerializer

        serializer = LanguageSerializer(data=language_provider.language())
        assert serializer.is_valid(raise_exception=True), "All provided fields should be valid"
        assert {"lang_info"} == set(serializer.validated_data)
        assert {"bidi", "code", "name", "name_local"} == set(serializer.validated_data.get("lang_info"))
        assert serializer.validated_data.get("lang_info").get("code") == "en"

    @pytest.mark.serializer
    @pytest.mark.django_db
    def test_deserialization_create(self, language_provider):
        """Check deserialization for ``Language`` model."""
        from translate.service.models import Language
        from translate.service.serializers import LanguageSerializer

        serializer = LanguageSerializer(data=language_provider.language())
        assert serializer.is_valid(raise_exception=True)

        obj, created = serializer.save()
        assert obj and obj.id

        obj_reloaded = Language.objects.get(id=obj.id)
        assert obj_reloaded and obj_reloaded.id
        assert obj_reloaded.lang_info == obj.lang_info


class TestTranslationKeySerializer:
    """Tests for ``TranslationKey`` model serializer."""

    @pytest.mark.serializer
    @pytest.mark.django_db
    def test_serialization_expected_fields(self, translation_key_factory):
        """Check expected fields after serializing ``TranslationKey`` model."""
        from translate.service.serializers import TranslationKeySerializer

        serializer = TranslationKeySerializer(translation_key_factory)
        for element in {"snake_name", "id", "id_name", "views"}:
            assert element in set(serializer.data)

        assert serializer.data.get("snake_name")
        assert isinstance(serializer.data.get("category"), str)
        assert serializer.data.get("category")
        assert serializer.data.get("usage_context") is None
        assert serializer.data.get("id")

    @pytest.mark.serializer
    @pytest.mark.django_db
    def test_deserialization_expected_fields(self, translation_key_provider):
        """Check model instance creation and validation through serializer."""
        from translate.service.serializers import TranslationKeySerializer

        serializer = TranslationKeySerializer(data=translation_key_provider.snake_key())

        assert serializer.is_valid(raise_exception=True)

        assert {"id_name", "category", "snake_name", "usage_context", "views"} == set(serializer.validated_data.keys())
        assert isinstance(serializer.validated_data.get("snake_name"), str)
        assert isinstance(serializer.validated_data.get("category"), int)
        assert isinstance(serializer.validated_data.get("usage_context"), str)
        assert isinstance(serializer.validated_data.get("id_name"), str)
        assert isinstance(serializer.validated_data.get("views"), list)

    @pytest.mark.serializer
    @pytest.mark.django_db
    def test_deserialization_create(self, translation_key_provider):
        """Check deserialization for ``TranslationKey``."""
        from translate.service.models import TranslationKey
        from translate.service.serializers import TranslationKeySerializer

        serializer = TranslationKeySerializer(data=translation_key_provider.snake_key())
        assert serializer.is_valid(raise_exception=True)

        obj, created = serializer.save()
        assert obj and obj.id

        obj_reloaded = TranslationKey.objects.get(id=obj.id)
        assert obj_reloaded and obj_reloaded.id


class TestTranslationSerializer:
    """Tests for ``Translation`` model serializer."""

    @pytest.mark.serializer
    @pytest.mark.django_db
    def test_serialization_expected_fields(self, translation_factory):
        """Check expected fields after serializing ``Translation`` model."""
        from translate.service.serializers import TranslationSerializer

        translation, *_ = translation_factory
        serializer = TranslationSerializer(translation)
        for element in {"translation", "key", "id"}:
            assert element in set(serializer.data)

        # NOTE: Check that read-only fields are populated as expected.
        assert serializer.data.get("id")
        assert serializer.data.get("translation")
        assert serializer.data.get("language")
        assert serializer.data.get("key")
        # NOTE: Assert that we didn't get any additional keys.
        assert set(serializer.data.keys()) == {"id", "language", "key", "translation"}

    @pytest.mark.serializer
    @pytest.mark.django_db
    def test_deserialization_expected_fields(self, translation_provider, language_factory, translation_key_factory):
        """Check deserialization"""
        from translate.service.serializers import TranslationSerializer

        serializer = TranslationSerializer(
            data=translation_provider.translation(language_factory, translation_key_factory)
        )

        assert serializer.is_valid(raise_exception=True)

        for element in {"key", "translation"}:
            assert element in set(serializer.validated_data.keys())

        assert isinstance(serializer.validated_data.pop("translation"), str)
        assert isinstance(serializer.validated_data.pop("language"), dict)
        assert isinstance(serializer.validated_data.pop("key"), dict)

    @pytest.mark.serializer
    @pytest.mark.django_db
    def test_deserialization_create(self, translation_provider, language_factory, translation_key_factory):
        """Check deserialization for ``TranslationSerializer``."""
        from translate.service.models import Translation
        from translate.service.serializers import TranslationSerializer

        data = translation_provider.translation(language_factory, translation_key_factory)

        serializer = TranslationSerializer(data=data)

        assert serializer.is_valid(raise_exception=True)

        obj, created = serializer.save()
        assert created
        assert obj and obj.id

        obj_reloaded = Translation.objects.get(id=obj.id)
        assert obj_reloaded and obj_reloaded.id and obj_reloaded.translation == obj.translation

        # cleanup of the test since the base object is being
        # created in the test itself not in fixture
        Translation.objects.filter(id=obj.id).delete()


class TestLivelinessCheckSerializer:
    """Tests for liveliness serializer."""

    def test_read(self):
        """Test reading default data."""
        from translate.service.serializers import LivelinessCheckSerializer

        assert LivelinessCheckSerializer().data == {"status": LivelinessCheckSerializer.LIVELINESS_DEFAULT_STATUS}


class TestReadinessCheckSerializer:
    """Tests for readiness serializer."""

    def test_read(self, readiness_provider):
        """Test reading default data."""
        from translate.service.serializers import ReadinessCheckSerializer

        data = ReadinessCheckSerializer(readiness_provider.health_check()).data

        assert data.pop("celery_backend") == "unavailable: Unknown error"
        assert data.pop("cache_backend") == "working"
        assert data.pop("database_backend") == "working"
        assert data.pop("fs_file_storage") == "working"
        assert data.pop("disk_usage") == "working"
        assert data.pop("memory_usage") == "working"
        assert data.pop("migrations_check") == "working"
        assert data.pop("redis_backend") == "working"

        # NOTE: Assert that only above values are contained.
        assert set(data) == set()


class TestRequestSerializer:
    """Test request serialization"""

    def test_request_serialization(self, request_serializer_fixture):
        serializer = request_serializer_fixture

        assert serializer.is_valid()
        assert serializer.validated_data
        assert isinstance(serializer.validated_data.get("language").get("code"), str)
        assert isinstance(serializer.validated_data.get("views"), list)
        assert isinstance(serializer.validated_data.get("occurrences"), list)
        assert isinstance(serializer.validated_data.get("snake_keys"), list)



from django.conf.locale import LANG_INFO
from django.db import transaction
from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from translate.service.models import Language, Translation, TranslationKey


class LanguageField(serializers.Field):
    """Custom language field serializer."""

    def to_internal_value(self, value) -> dict:
        """Override interface."""
        if not isinstance(value, str):
            raise ValidationError("invalid `lang_info` type")

        data = LANG_INFO.get(value)
        if not data:
            raise ValidationError("invalid `lang_info` - no metadata found")

        fallback = data.get("fallback")
        if fallback:
            data = LANG_INFO.get(fallback.pop())

        return data

    def to_representation(self, value):
        """Override interface"""
        return LANG_INFO.get(value)


class LanguageSerializer(serializers.ModelSerializer):
    """Language model serializer."""

    lang_info = LanguageField()

    class Meta:
        model = Language
        fields = [
            "id",
            "lang_info",
        ]

    def validate_lang_info(self, value: dict) -> dict:
        """Check lang_info and return metadata."""
        if not isinstance(value, dict) or not value.get("code"):
            raise ValidationError("invalid `lang_info` type")

        return value

    def validate(self, data) -> dict:
        """Object level validation."""
        return data

    def create(self, validated_data) -> Language:
        """Create and return a new ``Translation`` instance for a given validated data."""
        code = validated_data.get("lang_info").get("code")

        if not code or code not in LANG_INFO:
            raise ValidationError("invalid language locale")

        return Language.objects.get_or_create(lang_info=code)


class CategoryField(serializers.ChoiceField):
    """Custom field for Category Choices"""

    def to_representation(self, value):
        _choice = self._choices.get(value)
        if not value and self.allow_blank:
            raise ValidationError("Category value should be provided.")
        return _choice

    def to_internal_value(self, data):
        if not data:
            return None
        return data


class TranslationKeySerializer(serializers.ModelSerializer):
    """TranslationKey model serializer."""

    category = CategoryField(choices=TranslationKey.Category.choices, default=0)

    class Meta:
        model = TranslationKey
        fields = [
            "id",
            "usage_context",
            "category",
            "snake_name",
            "id_name",
            "views",
            "flags",
            "occurrences",
        ]

    def validate_snake_name(self, value) -> dict:
        """Check snake name."""
        if not value:
            raise ValidationError("invalid ``snake_name``")

        return value

    def validate(self, data) -> dict:
        """Object level validation."""
        return dict(data)

    def create(self, validated_data) -> "TranslationKey":
        """Create and return a new ``TranslationKey`` instance for a given validated data."""
        return TranslationKey.objects.get_or_create(**validated_data)


class TranslationSerializer(serializers.Serializer):
    """Translation model serializer."""

    id = serializers.UUIDField(read_only=True)
    language = LanguageSerializer()
    key = TranslationKeySerializer()
    translation = serializers.CharField()

    class Meta:
        model = Translation
        fields = [
            "id",
            "language",
            "key",
            "translation",
        ]

    def validate(self, data: dict) -> dict:
        """Object level validation."""
        return dict(data)

    def create(self, validated_data):
        """Create and return a new ``Translation`` instance for a given validated data."""
        translation_key = validated_data.pop("key")
        language = validated_data.pop("language").pop("lang_info")

        with transaction.atomic():
            key, _ = TranslationKey.objects.get_or_create(snake_name=translation_key.get("snake_name"))
            language, _ = Language.objects.get_or_create(lang_info=language.get("code"))

            return Translation.objects.get_or_create(
                language=language, key=key, translation=validated_data.get("translation")
            )


class TranslationKeySerializerV2(serializers.ModelSerializer):
    """TranslationKey model serializer."""

    class Meta:
        model = TranslationKey
        fields = [
            "id",
            "snake_name",
            "id_name",
            "views",
        ]

    def validate_snake_name(self, value) -> dict:
        """Check snake name."""
        if not value:
            raise ValidationError("invalid ``snake_name``")

        return value

    def validate(self, data) -> dict:
        """Object level validation."""
        return dict(data)

    def create(self, validated_data) -> "TranslationKey":
        """Create and return a new ``TranslationKey`` instance for a given validated data."""
        return TranslationKey.objects.get_or_create(**validated_data)


class TranslationSerializerV2(serializers.Serializer):
    """Translation model serializer."""

    id = serializers.UUIDField(read_only=True)
    key = TranslationKeySerializerV2()
    translation = serializers.CharField()

    class Meta:
        model = Translation
        fields = [
            "id",
            "key",
            "translation",
        ]

    def validate(self, data: dict) -> dict:
        """Object level validation."""
        return dict(data)


class TranslationRequestSerializer(serializers.Serializer):
    language = LanguageField(required=True)

    views = serializers.ListField(child=serializers.CharField(), required=False, default=[])
    occurrences = serializers.ListField(child=serializers.CharField(), required=False, default=[])
    snake_keys = serializers.ListField(child=serializers.CharField(), required=False, default=[])

    page = serializers.IntegerField(required=False, allow_null=True)
    page_size = serializers.IntegerField(required=False, allow_null=True)

    def validate(self, data) -> dict:
        """Object level validation."""
        return dict(data)


class MelonSerializer(serializers.Serializer):
    """Melon parent serializer."""

    def update(self, instance, validated_data):
        """Override default update field."""
        pass

    def create(self, validated_data):
        """Override default create field."""
        pass


class LivelinessCheckSerializer(MelonSerializer):
    """Liveliness check serializer."""

    LIVELINESS_DEFAULT_STATUS = "ok"

    status = serializers.CharField(initial=LIVELINESS_DEFAULT_STATUS)


class ReadinessCheckSerializer(MelonSerializer):
    """Readiness check serializer."""

    cache_backend = serializers.CharField(source="Cache backend: default", read_only=True)
    celery_backend = serializers.CharField(source="CeleryHealthCheckCelery", read_only=True)
    database_backend = serializers.CharField(source="DatabaseBackend", read_only=True)
    fs_file_storage = serializers.CharField(source="DefaultFileStorageHealthCheck", read_only=True)
    disk_usage = serializers.CharField(source="DiskUsage", read_only=True)
    memory_usage = serializers.CharField(source="MemoryUsage", read_only=True)
    migrations_check = serializers.CharField(source="MigrationsHealthCheck", read_only=True)
    redis_backend = serializers.CharField(source="RedisHealthCheck", read_only=True)



from django.core.management.base import BaseCommand

from translate.core.utils.logger import log
from translate.service.models import Translation, TranslationKey


class Command(BaseCommand):
    help = "Delete all keys that have no usage_context inserted"

    def handle(self, *args, **kwargs):
        translations = Translation.objects.filter(key__usage_context__isnull=True)
        t_keys = TranslationKey.objects.filter(usage_context__isnull=True)

        log.info(f"Prepared {len(translations)} Translations and {len(t_keys)} TranslationKeys for deletion.")
        log.info("Deleting...")

        translations.delete()
        t_keys.delete()

        log.info("Done!")



# Generated by Django 4.0.7 on 2022-09-16 09:49

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("service", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="translation",
            name="key",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.RESTRICT,
                related_name="translation_keys",
                to="service.translationkey",
            ),
        ),
        migrations.AlterField(
            model_name="translation",
            name="language",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.RESTRICT, related_name="translations", to="service.language"
            ),
        ),
    ]



import time
import uuid
from enum import unique

from auditlog.registry import auditlog
from django.conf.locale import LANG_INFO
from django.contrib.postgres.fields import ArrayField
from django.contrib.postgres.indexes import GinIndex
from django.db import models
from model_utils.models import TimeStampedModel, UUIDModel


class Language(TimeStampedModel, UUIDModel):
    """``Language`` records model."""

    lang_info = models.CharField(
        max_length=8,
        choices=[(lang, lang_info.get("name")) for lang, lang_info in LANG_INFO.items() if lang_info.get("name")],
        unique=True,
    )

    class Meta:
        verbose_name = "Language"
        verbose_name_plural = "Languages"

    def __str__(self):
        """String representation of ``Language``."""
        return LANG_INFO.get(self.lang_info, {}).get("name") or self.lang_info


def default_snake_name() -> str:
    """Generate default snake name."""
    # return f"snake_name_{uuid.uuid1(time.time()).hex}"
    return f"snake_name_{uuid.uuid1().hex}"


class TranslationKey(TimeStampedModel, UUIDModel):
    """``TranslationKey`` records model."""

    @unique
    class Category(models.IntegerChoices):
        """``TranslationKey`` categories."""

        GENERAL = 0, "General"
        WEB = 10, "Web app"
        MOBILE = 20, "Mobile app"
        SERVICE = 30, "Microservice"

    category = models.PositiveSmallIntegerField(default=Category.GENERAL, choices=Category.choices)

    snake_name = models.CharField(unique=True, max_length=4096, default=default_snake_name)
    encoding = models.CharField(max_length=16, default="utf-8")
    usage_context = models.TextField(default=None, null=True, blank=True)

    # TODO: We cannot enforce uniqueness on database level here,
    #  therefore we should add additional checks during serialization.
    id_name = models.TextField(default=None, null=True, blank=True)
    id_name_plural = models.TextField(default=None, null=True, blank=True)

    occurrences = ArrayField(models.CharField(max_length=1024), size=32, default=None, null=True, blank=True)
    flags = ArrayField(models.CharField(max_length=1024), size=32, default=None, null=True, blank=True)
    views = ArrayField(models.CharField(max_length=1024), size=4096, default=None, null=True, blank=True)

    class Meta:
        verbose_name = "Translation Key"
        verbose_name_plural = "Translation Keys"
        indexes = [
            GinIndex(fields=["occurrences", "views"], name="occ_views_gin", fastupdate=False),
        ]

    def __str__(self):
        """String representation of ``TranslationKey``."""
        return self.snake_name


class Translation(TimeStampedModel, UUIDModel):
    """``Translation`` records model."""

    language = models.ForeignKey(
        Language,
        on_delete=models.RESTRICT,
        related_name="translations",
    )
    key = models.ForeignKey(
        TranslationKey,
        on_delete=models.RESTRICT,
        to_field="id",
        related_name="translation_keys",
    )

    translation = models.TextField()
    translation_plural = models.TextField(default=None, null=True, blank=True)

    class Meta:
        verbose_name = "Translation"
        verbose_name_plural = "Translations"
        unique_together = (("language", "key"),)


auditlog.register(Translation)
auditlog.register(TranslationKey)
auditlog.register(Language)



import pytest
from django.urls import reverse
from rest_framework import status


class TestTranslationsView:
    """Test cases for translations View."""

    @pytest.mark.view
    @pytest.mark.django_db
    @pytest.mark.xdist_group(name="german_translations")
    def test_get(self, request_factory, import_german_translations_fixture):
        """Check view get method for getting all translations"""
        from translate.service.views import TranslationsAPIView

        kwargs = {"language": "de"}
        get_request = request_factory.get(reverse("api_translations", kwargs=kwargs))
        response = TranslationsAPIView.as_view()(get_request, **kwargs)

        assert response.status_code == status.HTTP_200_OK
        assert response.data


class TestLivelinessCheckView:
    """Test liveliness check view."""

    @pytest.mark.view
    def test_get(self, make_request):
        """Test liveliness check success for GET method."""

        from translate.service.views import LivelinessCheckView

        request = make_request(f"get::status_liveliness")
        response = LivelinessCheckView.as_view()(request)

        assert response and response.status_code == status.HTTP_200_OK
        assert set(response.data) == {"status"}


class TestReadinessCheckView:
    """Test readiness check view."""

    @pytest.mark.django_db
    def test_get(self, make_request):
        """Test readiness check success for GET method."""

        from translate.service.views import ReadinessCheckView

        request = make_request(f"get::status_readiness")
        response = ReadinessCheckView.as_view()(request)

        assert response and response.status_code == status.HTTP_200_OK
        assert set(response.data) == {
            "migrations_check",
            "database_backend",
            "fs_file_storage",
            "cache_backend",
        }



import glob
import json
from pathlib import Path

import polib
from django.conf import settings


def read_po_files(language_fixture) -> list:
    """Read po files."""
    translations_used = []
    # Construct the absolute fixture path
    fixture_dir = str(Path(__file__).parent.parent / f"tests/fixtures/{language_fixture}" / "**/*.po")

    po_files = [polib.pofile(po_file, encoding="utf-8") for po_file in glob.iglob(fixture_dir, recursive=True)]

    for po_file in po_files:
        for entry in po_file:
            if entry.msgid and entry.msgid not in translations_used:
                translations_used.append(entry.msgid)
            if entry.msgstr and entry.msgstr not in translations_used:
                translations_used.append(entry.msgstr)

    return translations_used


def read_json_files(directory, file_name) -> dict:
    """Read json files."""
    base_path = settings.BASE_DIR
    pattern = f"**/{directory}/{file_name}.json"
    file_path = next(base_path.glob(pattern), None)

    if file_path is None:
        raise FileNotFoundError(f"File not found for pattern: {pattern}")

    # Load the JSON data from the found file
    with open(file_path, "r", encoding="utf-8") as file:
        translations_data_used = json.load(file)

    return translations_data_used



import pytest
from django.core.management import call_command


class TestImportCommand:
    @pytest.mark.command
    @pytest.mark.django_db
    @pytest.mark.xdist_group(name="german_translations")
    def test_objects_creation(self, import_german_translations_fixture):
        """Check importing of translations."""
        from translate.service.models import Language, Translation, TranslationKey

        language = Language.objects.get(lang_info="de")
        translation_record = Translation.objects.filter(translation="Startdatum auf Webseite").first()
        key_record = TranslationKey.objects.get(snake_name="admin_landlord_request")

        assert language
        assert language.lang_info == "de"
        assert translation_record
        assert translation_record.key
        assert translation_record.language.lang_info == "de"
        assert key_record
        assert key_record.snake_name
        assert key_record.id_name
        assert key_record.Category

    @pytest.mark.command
    @pytest.mark.django_db
    @pytest.mark.xdist_group(name="german_translations")
    def test_command_idempotency(self, import_german_translations_fixture):
        """
        Test the re-execution of the command
        """
        from translate.service.models import Language, Translation, TranslationKey

        languages = Language.objects.all()
        translations = Translation.objects.all()
        keys = TranslationKey.objects.all()

        assert len(languages) == 1
        assert len(translations) == 9
        assert len(keys) == 9

        for key in keys:
            assert key.usage_context is None, "All imported keys should not have context"
            TranslationKey.objects.filter(id=key.id).update(usage_context="Add some new context")
            key.refresh_from_db()

        # recall the command to test if it will overwrite usage_context
        call_command("import_translations", translations_dir="translate/service/tests/fixtures/german_fixtures")

        reloaded_keys = TranslationKey.objects.all()
        for key in reloaded_keys:
            assert key.usage_context == "Add some new context", "Context should not be overwritten by same import"

        # recall of the command should not insert any new keys
        assert len(languages) == 1
        assert len(translations) == 9, "Translations number should stay the same"
        assert len(keys) == 9, "Keys number should stayed the same"


class TestKeysNoContextDeletion:
    @pytest.mark.command
    @pytest.mark.django_db
    @pytest.mark.xdist_group(name="german_translations")
    def test_objects_deletion(self, import_german_translations_fixture):
        """Check deletion of translation keys with no usage_context."""
        from translate.service.models import Translation, TranslationKey

        record = Translation.objects.all()
        key_record = TranslationKey.objects.all()

        assert len(record) == 9
        assert len(key_record) == 9

        for key in key_record:
            assert key.usage_context is None

        call_command("delete_no_context_keys")

        new_list_keys = list(TranslationKey.objects.all())

        assert len(new_list_keys) == 0, "Since no imported keys have usage_context, they should all be deleted"

    @pytest.mark.command
    @pytest.mark.django_db
    @pytest.mark.xdist_group(name="german_translations")
    def test_skipping_keys_with_context(self, import_german_translations_fixture):
        """Test not deleting key with usage_context"""
        from translate.service.models import Translation, TranslationKey

        record = Translation.objects.all()
        key_record = TranslationKey.objects.all()

        assert len(record) == 9
        assert len(key_record) == 9

        # Add usage context for every imported key
        for key in key_record:
            TranslationKey.objects.filter(id=key.id).update(usage_context="Add some new context")
            key.refresh_from_db()

        # call the command
        call_command("delete_no_context_keys")

        assert len(record) == 9, "Translation count should stay same after calling of command"
        assert len(key_record) == 9, "TranslationKeys count should stay same after calling of command"


class TestCSV:
    @pytest.mark.command
    @pytest.mark.django_db
    def test_export_csv_file_creation(self, export_csv_fixture):
        """Imports data into database and then export it as a CSV file"""
        import csv

        _CSV_HEADER = ["language", "key", "context", "translation", "translation_plural"]

        file_path = export_csv_fixture

        with file_path.open(mode="r") as csvfile:
            assert csvfile

            csv_dict = csv.DictReader(csvfile)
            assert csv_dict.fieldnames

            for field in csv_dict.fieldnames:
                assert field in _CSV_HEADER

            for item in csv_dict:
                assert item.get("language") == "de", "Language should be German for all keys"
                assert item.get("key"), "Key should always exist and be exported"
                assert item.get("translation"), "Translation should exist and be exported"

    @pytest.mark.command
    @pytest.mark.django_db
    def test_import_csv_command(self, import_french_translations_fixture):
        from pathlib import Path

        from django.core.management import call_command

        from translate.service.models import Translation, TranslationKey

        all_keys = TranslationKey.objects.all()
        all_translations = Translation.objects.all()

        assert len(all_keys) == 18
        assert len(all_translations) == 18

        existing_translations = {}

        for item in all_translations:
            existing_translations[item.key.snake_name] = item.translation

        assert existing_translations.get("admin_open_offers") == "offres ouvertes"
        assert existing_translations.get("general_projekts") == "Projet"
        assert existing_translations.get("admin_curator") == "Conservateur"
        assert existing_translations.get("general_tenant_type") == "Mietertyp"

        fixture = Path("translate/service/tests/fixtures/import_csv_test_fixture.csv")
        call_command("import_csv", fixture)

        all_keys_updated = TranslationKey.objects.all()
        all_translations_updated = Translation.objects.all()

        translations_updated = {}

        for item in all_translations_updated:
            translations_updated[item.key.snake_name] = item.translation

        assert len(all_keys_updated) == 18, "Import command should not add new keys, only update existing ones"
        assert (
            len(all_translations_updated) == 18
        ), "Import command should not add new translations, only update existing ones"
        assert existing_translations != translations_updated
        assert translations_updated.get("admin_open_offers") == "CSV update testing"
        assert translations_updated.get("general_projekts") == "Projet testing 2"
        assert translations_updated.get("admin_curator") == "Conservateur 3"
        assert translations_updated.get("general_tenant_type") == "_TESTPerformances_"

    @pytest.mark.command
    @pytest.mark.django_db
    def test_bulk_import_translations_from_csv(self, bulk_import_translations_from_csv_fixture):
        """Test for bulk_import_translations_from_csv management command.
        Command should create new keys and translations from csv file."""
        from translate.service.models import Translation, TranslationKey

        all_keys = TranslationKey.objects.all()

        assert len(all_keys) == 5, "5 keys should be created from csv file"

        for key in all_keys:
            assert key.snake_name in [
                "general_password",
                "general_project_creation",
                "admin_open_offers_rejected",
                "admin_history",
                "general_test_performances",
            ], "Keys should be created from csv file"

        all_translations = Translation.objects.all()

        for trans in all_translations:
            assert trans.translation in [
                "L'utilisateur n'existe pas",
                "Project",
                "CSV creation testing",
                "History",
                "_TESTPerformances_",
            ]



import csv
from pathlib import Path

from django.conf.locale import LANG_INFO
from django.core.exceptions import ObjectDoesNotExist
from django.core.management.base import BaseCommand

from translate.core.utils.logger import log
from translate.service.models import Language, Translation, TranslationKey


class Command(BaseCommand):
    """
    Updates translations in database from CSV file. It does not create new Translation objs, nor TranslationKey objs.
    Languages need to be created before usage of the command.
    Following format of CSV file is accepted:  language, key, context, translation, translation_plural
    """

    help = "Updates database with translations from CSV file"

    def add_arguments(self, parser):
        """Attach argument for import_translations command."""

        parser.add_argument(
            "file_name",
            type=str,
            nargs=1,
            default="translations_de.csv",
        )

    @staticmethod
    def get_language(file_name):
        """Opens CSV file and returns language obj"""
        file_name = Path(file_name)
        with file_name.open() as csvfile:
            reader_dict = csv.DictReader(csvfile)
            field_name = reader_dict.fieldnames[0]
            for record in reader_dict:
                language_code = record.get(field_name)
                if language_code not in LANG_INFO:
                    raise RuntimeError(f"Language {language_code} not found.")
                try:
                    language = Language.objects.get(lang_info=language_code)
                    return language
                except ObjectDoesNotExist:
                    log.error(f"Language {language_code} not found in database.")
                    break

    @staticmethod
    def import_csv(language: Language, file_name):
        """Import translations from CSV file to database."""

        file_name = Path(file_name)
        log.info(f"Importing translations from {file_name.absolute()}")
        not_created = []

        with file_name.open() as csvfile:
            reader_dict = csv.DictReader(csvfile)

            for record in reader_dict:
                snake_key = record.get("key")
                translation = record.get("translation")  # .encode("utf-8").decode("ISO-8859-1")
                translation_plural = record.get("translation_plural")  # .encode("utf-8").decode("ISO-8859-1")

                if translation and translation_plural:
                    try:
                        key = TranslationKey.objects.get(snake_name=snake_key)
                    except ObjectDoesNotExist:
                        log.info(f"This key {snake_key} does not exists in database...")
                        not_created.append(snake_key)
                        continue

                    translation_updated = Translation.objects.filter(language=language, key=key).update(
                        translation=translation, translation_plural=translation_plural
                    )
                    if translation_updated:
                        log.info(f"Updating record {snake_key} for {language.lang_info} language")
        return not_created

    def handle(self, *args, **options):
        """Handle command."""
        file_name = options.get("file_name")[0]

        language = self.get_language(file_name)
        not_created = self.import_csv(language, file_name)

        if not_created:
            log.warning(
                f"Some keys have not been previously created: {not_created}"
                "Have you created keys with import_translations management command?"
            )

        log.info("Done!")



# Generated by Django 4.0.8 on 2022-11-23 15:26

import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("service", "0002_alter_translation_key_alter_translation_language"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="translationkey",
            index=django.contrib.postgres.indexes.GinIndex(
                fastupdate=False, fields=["occurrences", "views"], name="occ_views_gin"
            ),
        ),
    ]



from baton.autodiscover import admin
from django.urls import include, path
from drf_spectacular.views import SpectacularAPIView, SpectacularRedocView

from translate.service.views import HomeView

urlpatterns = [
    path("", HomeView.as_view(), name="home"),
    path("admin/", admin.site.urls, name="admin"),
    path("baton/", include("baton.urls")),
    path("api/", include("translate.service.urls")),
    path("api/schema/", SpectacularAPIView.as_view(), name="schema"),
    path("api/schema/redoc/", SpectacularRedocView.as_view(url_name="schema"), name="redoc"),
    path("health/", include("health_check.urls")),
]



import pytest
from django.conf.locale import LANG_INFO


class TestTranslationModel:
    """Tests for ``Translation`` model."""

    @pytest.mark.model
    @pytest.mark.django_db
    def test_translation_create(self, translation_factory):
        """Check creation of translation record."""
        from translate.service.models import Translation

        translation, language, translation_key = translation_factory

        # NOTE: Reload from database to ensure that record is stored.
        record = Translation.objects.get(id=translation.id)
        assert translation.translation == record.translation

        assert record.key.snake_name
        assert record.language.lang_info in LANG_INFO

    @pytest.mark.model
    @pytest.mark.django_db
    def test_language_create(self, language_factory):
        """Check creation of language record."""
        from translate.service.models import Language

        # NOTE: Reload from database to ensure that record is stored.
        record = Language.objects.get(id=language_factory.id)
        assert record.lang_info in LANG_INFO

    @pytest.mark.model
    @pytest.mark.django_db
    def test_translation_key_create(self, translation_key_factory):
        """Check creation of language key record."""
        from translate.service.models import TranslationKey as Tk

        # NOTE: Reload from database to ensure that record is stored.
        record = Tk.objects.get(id=translation_key_factory.id)

        assert record.snake_name
        assert record.category == Tk.Category.SERVICE.value
        assert record.usage_context is None



import uuid
from typing import Any, Dict

from faker.providers import BaseProvider


class TranslationProvider(BaseProvider):
    """Data provider for ``TranslationSerializer``."""

    def translation(self, language_factory, translation_key_factory) -> Dict[str, str]:
        """Translation data instance."""
        return {
            "language": {"lang_info": language_factory.lang_info},
            "key": {"snake_name": uuid.uuid1().hex, "category": 30},
            "translation": "my little translation",
        }


class LanguageProvider(BaseProvider):
    """Data provider for ``LanguageSerializer``."""

    def language(self) -> Dict[str, str]:
        """Language data instance."""
        return {
            "lang_info": "en",
        }


class TranslationKeyProvider(BaseProvider):
    """Data provider for ``TranslationKeySerializer``."""

    def snake_key(self) -> Dict[str, Any]:
        """Translation key data instance."""
        return {
            "snake_name": str(self.generator.name()).replace(" ", "_"),
            "category": 30,
            "usage_context": "context",
            "id": uuid.uuid4().hex,
            "id_name": "some id name",
            "views": ["TRANSLATION_CENTER_FRONTEND"],
        }


class ReadinessCheckProvider(BaseProvider):
    """Readiness response provider."""

    @staticmethod
    def health_check() -> dict:
        """Django health check provider."""
        data = {
            "Cache backend: default": "working",
            "CeleryHealthCheckCelery": "unavailable: Unknown error",
            "DatabaseBackend": "working",
            "DefaultFileStorageHealthCheck": "working",
            "DiskUsage": "working",
            "MemoryUsage": "working",
            "MigrationsHealthCheck": "working",
            "RedisHealthCheck": "working",
        }
        return data



import json

from django.db.models import Q
from django.views.generic import TemplateView
from health_check.views import MainView
from rest_framework import status
from rest_framework.generics import GenericAPIView, ListAPIView
from rest_framework.response import Response

from translate.service.models import Translation
from translate.service.paginations import TranslateClientPagination
from translate.service.serializers import (
    LivelinessCheckSerializer,
    ReadinessCheckSerializer,
    TranslationRequestSerializer,
    TranslationSerializer,
    TranslationSerializerV2,
)


class TranslationsAPIView(ListAPIView):
    """
    This view should return a list of translations
    for provided language
    """

    serializer_class = TranslationSerializer
    pagination_class = TranslateClientPagination
    queryset = Translation.objects

    def get(self, request, *args, **kwargs):
        data = {
            "language": kwargs.get("language", None),
            "views": self.request.GET.getlist("views"),
            "occurrences": self.request.GET.getlist("occurrences"),
            "snake_keys": self.request.GET.getlist("snake_keys"),
            "page": self.request.GET.get("page"),
            "page_size": self.request.GET.get("page_size"),
        }

        req_serializer = TranslationRequestSerializer(data=data)

        req_serializer.is_valid(raise_exception=True)
        validated_params = req_serializer.validated_data

        query = Q(language__lang_info=validated_params.get("language").get("code"))

        view_names = validated_params.get("views")
        if view_names:
            query &= Q(key__views__contains=view_names)

        occurrences = validated_params.get("occurrences")
        if occurrences:
            query &= Q(key__occurrences__contains=occurrences)

        snake_keys = validated_params.get("snake_keys")
        if snake_keys:
            query &= Q(key__snake_name__in=snake_keys)

        qs = self.queryset.filter(query).order_by("-modified")
        data = self.paginate_queryset(qs)

        tran_serializer = TranslationSerializer(data, many=True)
        paginated_response = self.get_paginated_response(tran_serializer.data)

        return paginated_response


class TranslationsAPIViewV2(ListAPIView):
    """
    This view should return a list of translations
    for provided language
    """

    serializer_class = TranslationSerializerV2
    pagination_class = TranslateClientPagination
    queryset = Translation.objects

    def get(self, request, *args, **kwargs):

        data = {
            "language": kwargs.get("language", None),
            "views": self.request.GET.getlist("views"),
            "snake_keys": self.request.GET.getlist("snake_keys"),
            "page": self.request.GET.get("page"),
            "page_size": self.request.GET.get("page_size"),
        }

        req_serializer = TranslationRequestSerializer(data=data)

        req_serializer.is_valid(raise_exception=True)
        validated_params = req_serializer.validated_data

        query = Q(language__lang_info=validated_params.get("language").get("code"))

        view_names = [item for item in validated_params.get("views") if item and item.strip() != ""]
        if view_names:
            _query = Q()
            for q_object in view_names:
                _query |= Q(key__views__contains=[q_object])
            query &= _query
            query &= Q(key__views__isnull=False)

        qs = self.queryset.filter(query).order_by("-modified")
        data = self.paginate_queryset(qs)

        tran_serializer = TranslationSerializerV2(data, many=True)
        paginated_response = self.get_paginated_response(tran_serializer.data)

        return paginated_response


class LivelinessCheckView(GenericAPIView):
    """Liveliness check view."""

    serializer_class = LivelinessCheckSerializer

    def get(self, request, *args, **kwargs):
        """Returns liveliness check response."""
        return Response(self.get_serializer().data, status=status.HTTP_200_OK)


class ReadinessCheckView(MainView, GenericAPIView):
    """Readiness check view."""

    serializer_class = ReadinessCheckSerializer

    def get(self, request, *args, **kwargs):
        """Return readiness check response."""
        request.GET = request.GET.copy()
        request.GET["format"] = "json"

        parent = super().get(request, *args, **kwargs)
        data = json.loads(parent.getvalue())

        return Response(self.get_serializer(data).data, status=parent.status_code)


class HomeView(TemplateView):
    template_name = "index.html"



import csv
from pathlib import Path

from django.core.management.base import BaseCommand

from translate.core.utils.logger import log
from translate.service.models import Language, Translation, TranslationKey


class Command(BaseCommand):
    """
    Creates or updates translation keys in database from CSV file.
    Languages need to be created before usage of the command.
    Following format of CSV file is accepted:
    key,views, context, translation_de, translation_plural_de, translation_en, translation_plural_en...
    """

    def add_arguments(self, parser):
        """Attach argument for import_translations command."""

        parser.add_argument(
            "file_name",
            type=str,
            nargs=1,
        )

    @staticmethod
    def get_lang_dict():
        lang_dict = {}
        for lang in Language.objects.all():
            lang_dict[lang.lang_info] = lang
        return lang_dict

    def import_csv(self, file_name):
        """Import translations from CSV file to database."""

        file_name = Path(file_name)
        language_dict = self.get_lang_dict()
        if not language_dict:
            log.warning("No languages found in database. Please create languages first.")
            return
        log.info(f"Importing translations from {file_name.absolute()}")

        with file_name.open() as csvfile:
            reader_dict = csv.DictReader(csvfile)

            for record in reader_dict:
                snake_key = record.get("key")
                views = record.get("views").split(":")
                context = record.get("context")
                translation_key, _ = TranslationKey.objects.get_or_create(
                    snake_name=snake_key, defaults={"views": views, "usage_context": context}
                )

                for lang_code, lang in language_dict.items():
                    translation = record.get(f"translation_{lang_code}")
                    translation_plural = record.get(f"translation_plural_{lang_code}")

                    if not translation:
                        continue

                    translation_obj, created = Translation.objects.update_or_create(
                        language=lang,
                        key=translation_key,
                        defaults={"translation": translation, "translation_plural": translation_plural},
                    )
                    if created:
                        log.info(f"Creating record {snake_key} for {lang.lang_info} language")
                    else:
                        log.info(f"Updating record {snake_key} for {lang.lang_info} language")

    def handle(self, *args, **options):
        """Handle command."""
        file_name = options.get("file_name")[0]

        self.import_csv(file_name)

        log.info("Done!")



# Generated by Django 4.0.4 on 2022-05-27 12:45

import uuid

import django.contrib.postgres.fields
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
from django.db import migrations, models

import translate.service.models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Language",
            fields=[
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                (
                    "id",
                    model_utils.fields.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                (
                    "lang_info",
                    models.CharField(
                        choices=[
                            ("af", "Afrikaans"),
                            ("ar", "Arabic"),
                            ("ar-dz", "Algerian Arabic"),
                            ("ast", "Asturian"),
                            ("az", "Azerbaijani"),
                            ("be", "Belarusian"),
                            ("bg", "Bulgarian"),
                            ("bn", "Bengali"),
                            ("br", "Breton"),
                            ("bs", "Bosnian"),
                            ("ca", "Catalan"),
                            ("cs", "Czech"),
                            ("cy", "Welsh"),
                            ("da", "Danish"),
                            ("de", "German"),
                            ("dsb", "Lower Sorbian"),
                            ("el", "Greek"),
                            ("en", "English"),
                            ("en-au", "Australian English"),
                            ("en-gb", "British English"),
                            ("eo", "Esperanto"),
                            ("es", "Spanish"),
                            ("es-ar", "Argentinian Spanish"),
                            ("es-co", "Colombian Spanish"),
                            ("es-mx", "Mexican Spanish"),
                            ("es-ni", "Nicaraguan Spanish"),
                            ("es-ve", "Venezuelan Spanish"),
                            ("et", "Estonian"),
                            ("eu", "Basque"),
                            ("fa", "Persian"),
                            ("fi", "Finnish"),
                            ("fr", "French"),
                            ("fy", "Frisian"),
                            ("ga", "Irish"),
                            ("gd", "Scottish Gaelic"),
                            ("gl", "Galician"),
                            ("he", "Hebrew"),
                            ("hi", "Hindi"),
                            ("hr", "Croatian"),
                            ("hsb", "Upper Sorbian"),
                            ("hu", "Hungarian"),
                            ("hy", "Armenian"),
                            ("ia", "Interlingua"),
                            ("io", "Ido"),
                            ("id", "Indonesian"),
                            ("ig", "Igbo"),
                            ("is", "Icelandic"),
                            ("it", "Italian"),
                            ("ja", "Japanese"),
                            ("ka", "Georgian"),
                            ("kab", "Kabyle"),
                            ("kk", "Kazakh"),
                            ("km", "Khmer"),
                            ("kn", "Kannada"),
                            ("ko", "Korean"),
                            ("ky", "Kyrgyz"),
                            ("lb", "Luxembourgish"),
                            ("lt", "Lithuanian"),
                            ("lv", "Latvian"),
                            ("mk", "Macedonian"),
                            ("ml", "Malayalam"),
                            ("mn", "Mongolian"),
                            ("mr", "Marathi"),
                            ("ms", "Malay"),
                            ("my", "Burmese"),
                            ("nb", "Norwegian Bokmal"),
                            ("ne", "Nepali"),
                            ("nl", "Dutch"),
                            ("nn", "Norwegian Nynorsk"),
                            ("no", "Norwegian"),
                            ("os", "Ossetic"),
                            ("pa", "Punjabi"),
                            ("pl", "Polish"),
                            ("pt", "Portuguese"),
                            ("pt-br", "Brazilian Portuguese"),
                            ("ro", "Romanian"),
                            ("ru", "Russian"),
                            ("sk", "Slovak"),
                            ("sl", "Slovenian"),
                            ("sq", "Albanian"),
                            ("sr", "Serbian"),
                            ("sr-latn", "Serbian Latin"),
                            ("sv", "Swedish"),
                            ("sw", "Swahili"),
                            ("ta", "Tamil"),
                            ("te", "Telugu"),
                            ("tg", "Tajik"),
                            ("th", "Thai"),
                            ("tk", "Turkmen"),
                            ("tr", "Turkish"),
                            ("tt", "Tatar"),
                            ("udm", "Udmurt"),
                            ("uk", "Ukrainian"),
                            ("ur", "Urdu"),
                            ("uz", "Uzbek"),
                            ("vi", "Vietnamese"),
                            ("zh-hans", "Simplified Chinese"),
                            ("zh-hant", "Traditional Chinese"),
                        ],
                        max_length=8,
                        unique=True,
                    ),
                ),
            ],
            options={
                "verbose_name": "Language",
                "verbose_name_plural": "Languages",
            },
        ),
        migrations.CreateModel(
            name="TranslationKey",
            fields=[
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                (
                    "id",
                    model_utils.fields.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                (
                    "category",
                    models.PositiveSmallIntegerField(
                        choices=[(0, "General"), (10, "Web app"), (20, "Mobile app"), (30, "Microservice")], default=0
                    ),
                ),
                (
                    "snake_name",
                    models.CharField(default=translate.service.models.default_snake_name, max_length=4096, unique=True),
                ),
                ("encoding", models.CharField(default="utf-8", max_length=16)),
                ("usage_context", models.TextField(blank=True, default=None, null=True)),
                ("id_name", models.TextField(blank=True, default=None, null=True)),
                ("id_name_plural", models.TextField(blank=True, default=None, null=True)),
                (
                    "occurrences",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(max_length=1024), blank=True, default=None, null=True, size=32
                    ),
                ),
                (
                    "flags",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(max_length=1024), blank=True, default=None, null=True, size=32
                    ),
                ),
                (
                    "views",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(max_length=1024), blank=True, default=None, null=True, size=4096
                    ),
                ),
            ],
            options={
                "verbose_name": "Translation Key",
                "verbose_name_plural": "Translation Keys",
            },
        ),
        migrations.CreateModel(
            name="Translation",
            fields=[
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                (
                    "id",
                    model_utils.fields.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                ("translation", models.TextField()),
                ("translation_plural", models.TextField()),
                (
                    "key",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translation_keys",
                        to="service.translationkey",
                    ),
                ),
                (
                    "language",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, related_name="translations", to="service.language"
                    ),
                ),
            ],
            options={
                "verbose_name": "Translation",
                "verbose_name_plural": "Translations",
                "unique_together": {("language", "key")},
            },
        ),
    ]



import re
from pathlib import Path

import django
from django.conf.locale import LANG_INFO
from django.core.exceptions import ObjectDoesNotExist
from django.core.management.base import BaseCommand
from django.utils.translation import gettext as _

from translate.core.utils.logger import log
from translate.service.models import Language, Translation, TranslationKey

CUSTOM_OCCURRENCES = "new_mdt/custom_translation.py"
FORM_FIELD_OCCURRENCES = "new_mdt/form_field_translation.py"

CUSTOM_TRANSLATION = [
    _("This field is required."),
    _("Previous"),
    _("Next"),
    _("is_advertised"),
    _("min_occupancy"),
    _("apartment"),
    _("max_adult"),
    _("min_adult"),
    _("max_child"),
    _("min_child"),
    _("image"),
    _("pdf_file"),
    _("capital_share"),
    _("vacancies"),
    _("tender_notice"),
    _("building"),
    _("assignment_direct"),
    _("create_advert"),
    _("min_occupancy_info"),
    _("num_applications"),
    _("reference_date"),
    _("rentalgross"),
    _("rentalgross_net"),
    _("sel"),
    _("subsidized"),
    _("all_offers"),
    _("assignment_apartment"),
    _("assignment_other"),
    _("assignment_parkingspace"),
    _("children"),
    _("evaluation_overall_comment"),
    _("evaluation_priority"),
    _("korrespondenz_per_post"),
    _("mail_history"),
    _("modiftrue"),
    _("number_of_declined"),
    _("number_of_declined_tenant"),
    _("modified"),
    _("createdUG"),
    _("relocationrequest"),
    _("proposions"),
    _("number_of_uninterested_tenant"),
    _("balcony"),
    _("parterre"),
    _("elevator"),
    _("number_of_offers"),
    _("prio_ids"),
    _("prios"),
    _("subvention_income"),
    _("subvention_wealth"),
    _("subvention_residence"),
    _("voucher_code"),
    _("recall"),
    _("assignment_detail"),
    _("assignment_parking"),
    _("assignment_state"),
    _("contact_tel"),
    _("umzugsgrund"),
    _("created"),
    _("befristed"),
    _("assignment_single"),
    _("ref"),
    _("application_id"),
    _("employer"),
    _("enforcement_doc"),
    _("in_education"),
    _("Confirmation"),
    _("Mail Signature"),
    _("Postmark Sender Name"),
    _("sum_bicycles"),
    _("history"),
    _("parking"),
    _("mail history"),
    _("actions"),
    _("property_type"),
    _("state"),
    _("Create"),
    _("Cooperative Experience"),
    _("You have not completed all mandatory fields"),
    _("Application"),
    _("living_since"),
    _("previous_rent"),
    _("incidental_costs"),
    _("income_gros"),
    _("company_since"),
    _("Address"),
    _("valid"),
    _("area"),
    _("notice_of_termination"),
    _("Create Apartment"),
    _("Is Advertised"),
    _("Additional Room For Prio 1"),
    _("Additional Room For Prio 2"),
    _("Additional Room For Prio 3"),
    _("Hobbyroom"),
    _("Household Structure"),
    _("Applicants"),
    _("Rental Properties"),
    _("Mobility"),
    _("Additional"),
    _("Send Application"),
    _("Contract"),
    _("All assigments"),
    _("Assignment Parking"),
    _("points"),
    _("Num Children"),
    _("Additional Room"),
    _("Parking"),
    _("false"),
    _("true"),
    _("street nr"),
    _("city"),
    _("moving date"),
    _("solvent"),
    _("wants hobbyroom"),
    _("evaluation priority"),
    _("assignment"),
    _("agreement references"),
    _("active prio title temp"),
    _("occupation"),
    _("date_of_birth"),
    _("postcode"),
    _("company_phone"),
    _("firstname"),
    _("member_since"),
    _("income_gross"),
    _("membership_id"),
    _("company"),
    _("company_postcode"),
    _("legal_residence"),
    _("nation"),
    _("email"),
    _("civil_status"),
    _("company_city"),
    _("landlordphone"),
    _("office_phone"),
    _("phone"),
    _("member"),
    _("permit"),
    _("highest_education"),
    _("street_nr"),
    _("same_address"),
    _("adress"),
    _("same_address"),
    _("days_present"),
    _("subventions_tax"),
    _("adress_2yago"),
    _("overall_evaluation"),
    _("evaluation_overall_star"),
    _("True"),
    _("False"),
    _("household_social_mix"),
    _("income_comment"),
    _("household_second_evaluation"),
    _("income_needs_clarfication"),
    _("instrument_needs_clarification"),
    _("instrument_comment"),
    _("instrument_second_evaluation"),
    _("references_needs_clarification"),
    _("references_second_evaluation"),
    _("references_ok"),
    _("documents_comment"),
    _("documents_second_evaluation"),
    _("documents_ok"),
    _("finance_needs_clarification"),
    _("finance_comment"),
    _("finance_ok"),
    _("validity_needs_clarification"),
    _("validity_comment"),
    _("validity_second_evaluation"),
    _("validity_ok"),
    _("experience_ok"),
    _("occupancy_comment"),
    _("occupancy_needs_clarification"),
    _("occupancy_prio_1"),
    _("occupancy_prio_2"),
    _("occupancy_prio_3"),
    _("occupancy_second_evaluation"),
    _("overall_evaluation_second_evaluator"),
    _("overall_evaluation_comment"),
    _("overall_evaluation_done"),
    _("overall_evaluation_second_evaluator_done"),
    _("overall_needs_clarification"),
    _("overall_comment"),
    _("parking_ok"),
    _("pets_needs_clarification"),
    _("pets_comment"),
    _("pets_second_evaluation"),
    _("solvency_prio_1"),
    _("solvency_prio_2"),
    _("solvency_prio_3"),
    _("solvency_comment"),
    _("solvency_needs_clarification"),
    _("solvency_second_evaluation"),
    _("solvency_financial_conditions"),
    _("mobility_traffic_behavior"),
    _("mobility_second_evaluation"),
    _("mobility_needs_clarification"),
    _("mobility_ok"),
    _("cooperative_experience"),
    _("cooperative_commitment"),
    _("residency_rights_values"),
    _("experience_needs_clarification"),
    _("experience_comment"),
    _("experience_second_evaluation"),
    _("experience_comment"),
    _("experience_second_evaluation"),
    _("commitment_comment"),
    _("commitment_needs_clarification"),
    _("commitment_second_evaluation"),
    _("further_information_comment"),
    _("further_information_second_evaluation"),
    _("further_information_needs_clarification"),
    _("presentation_comment"),
    _("presentation_impression"),
    _("car_evaluation"),
    _("presentation_attended"),
    _("presentation_invited"),
    _("urgency"),
    _("neighborhood"),
    _("general_commitment"),
    _("future_commitment"),
    _("living_and_working"),
    _("second_evaluator"),
    _("dog"),
    _("income_needs_clarification"),
    _("finance_second_evaluation"),
    _("recall_date"),
    _("relocation_reason"),
    _("number_ebicycles"),
    _("workshop_intended_use"),
    _("engagement"),
    _("num_ecars"),
    _("number_bicycles"),
    _("source_reason"),
    _("addroom"),
    _("source_reason"),
    _("numcars"),
    _("pets_detail"),
    _("remarks"),
    _("agreement_references"),
    _("hobbyroom_intended_use"),
    _("household_type"),
    _("number_moped"),
    _("parking_num_cars"),
    _("source"),
    _("wants_hobbyroom"),
    _("wants_workshop"),
    _("nummotorbikes"),
    _("ug_status"),
    _("actualisation"),
    _("userconfirmation"),
    _("userinactive"),
    _("parking_num_cars"),
    _("employment_quota"),
    _("Garage"),
    _("on"),
    _("references_comment"),
    _("country"),
    _("documents_needs_clarification"),
    _("enforcement_doc_mail"),
    _("Additional Room"),
    _("send_mail"),
    _("denied_mail"),
    _("denied_no_prios"),
    _("reminder"),
    _("tenant_info"),
    _("reminder"),
    _("relocation_confirmation_link"),
    _("relocation_confirmation_without_link"),
    _("relocation_offer_link"),
    _("relocation_offer_without_link"),
    _("relocation_acceptance"),
    _("relocation_declining"),
    _("Enforcement OK"),
    _("Landlord Contact"),
    _("Landlord Payment"),
    _("Landlord Terminated"),
    _("Landlord Terminated Date"),
    _("Work Date"),
    _("Employed By"),
    _("Work Since"),
    _("Work Terminated"),
    _("Work Terminated Date"),
    _("Work Salary"),
    _("Permit OK"),
    _("Needs Evaluation"),
    _("Design"),
    _("employee"),
    _("evaluator"),
    _("manager"),
    _("team_leader"),
    _("user_read"),
    _("user_write"),
    _("applications_read"),
    _("applications_write"),
    _("evaluations_read"),
    _("evaluations_write"),
    _("masterdata_read"),
    _("masterdata_write"),
    _("assignments_read"),
    _("assignments_write"),
    _("analytics_read"),
    _("insurance_read"),
    _("axa_client"),
    _("insurance_private"),
    _("axa_client_id"),
    _("insurance_home"),
    _("insurance_agent"),
    _("admin_read"),
    _("export_data"),
    _("send_mails"),
    _("get_references"),
    _("config_view"),
    _("execute_random"),
    _("change_state"),
    _("import_data"),
    _("offer_read"),
    _("objectdata_read"),
    _("analytics_write"),
    _("admin_write"),
    _("newtenant_offer_link"),
    _("newtenant_offer_without_link"),
    _("newtenant_acceptance"),
    _("send_offer_denied"),
    _("send_offer_accepted"),
    _("offer_reminder"),
    _("newtenant_confirmation_without_link"),
    _("newtenant_confirmation_link"),
    _("newtenant_declining"),
    _("newtenant_declining_no_prios_without_link"),
    _("newtenant_declining_no_prios_link"),
    _("to_6"),
    _("to_12"),
    _("to_18"),
    _("to_24"),
    _("to_30"),
    _("to_36"),
    _("to_180"),
    _("move in date"),
    _("num ecars"),
    _("number bicycles"),
    _("number ebicycles"),
    _("music instruments"),
    _("wants mobility"),
    _("relocation reason"),
    _("subvention residence since"),
    _("source reason"),
    _("apartment visited"),
    _("household type"),
    _("subvention wealth"),
    _("subvention income"),
    _("parking num cars"),
    _("stockroom intended use"),
    _("pets detail"),
    _("car reason"),
    _("hobbyroom intended use"),
    _("source web"),
    _("addroom intended use"),
    _("music instruments type"),
    _("agreement deposit"),
    _("workshop intended use"),
    _("car documentation"),
    _("motorbikes"),
    _("securities"),
    _("number cycle trailer"),
    _("motivation"),
    _("family apartment"),
    _("number moped"),
    _("insurance"),
    _("deposit"),
    _("is advertised"),
    _("in education"),
    _("company since"),
    _("income gross"),
    _("membership id"),
    _("notice of termination"),
    _("application id"),
    _("office phone"),
    _("subventions tax"),
    _("legal residence"),
    _("company city"),
    _("living since"),
    _("adress 2yago"),
    _("enforcement"),
    _("living arrangements"),
    _("rental"),
    _("member since"),
    _("tenant type"),
    _("civil status"),
    _("enforcement doc"),
    _("company postcode"),
    _("previous rent"),
    _("company phone"),
    _("employment quota"),
    _("Validity"),
    _("enforcement monthly rate"),
    _("permit doc"),
    _("sum bicycles"),
    _("voucher code"),
    _("evaluation overall comment"),
    _("max adult"),
    _("capital share"),
    _("tender notice"),
    _("min adult"),
    _("pdf file"),
    _("max child"),
    _("min child"),
    _("Apartment"),
    _("Parkingspace"),
    _("Other"),
    _("EvaluationEvaluatorsForm"),
    _("EvaluationHouseholdForm"),
    _("EvaluationObjectsForm"),
    _("EvaluationSolvencyForm"),
    _("EvaluationMobilityForm"),
    _("EvaluationEmployerReferencesForm"),
    _("EvaluationLandlordReferencesForm"),
    _("EvaluationInstrumentsForm"),
    _("EvaluationFinanceForm"),
    _("EvaluationAnimalsForm"),
    _("EvaluationFurtherInformationForm"),
    _("EvaluationDocumentsForm"),
    _("EvaluationSummaryForm"),
    _("EvaluationEventForm"),
    _("SecondEvaluationSummaryForm"),
    _("EvaluationValidityForm"),
    _("EvaluationApplicantCommentForm"),
    _("EvaluationMotivationForm"),
    _("EvaluationPriorityForm"),
    _("EvaluationSubsidyForm"),
    _("EvaluationExperienceForm"),
    _("EvaluationCommitmentForm"),
    _("active.in_progress"),
    _("active.needs_discussion"),
    _("active.new"),
    _("active.evaluated"),
    _("active.new_customer"),
    _("vacant"),
    _("with_applications"),
    _("without_applications"),
    _("parkingspace"),
    _("object_type"),
    _("object type"),
    _("other"),
    _("active.open"),
    _("active.sent"),
    _("final"),
    _("living_arrangements"),
    _("tenant_type"),
    _("occupancy_evaluation_prio_1"),
    _("overall_flag"),
    _("overall_rejection_reason"),
    _("overall_star"),
    _("household_evaluation"),
    _("household_flag"),
    _("household_rejection_reason"),
    _("instrument_evaluation"),
    _("instrument_flag"),
    _("instrument_rejection_reason"),
    _("landlord_references_flag"),
    _("landlord_references_rejection_reason"),
    _("landlord_references_needs_clarification"),
    _("employer_references_flag"),
    _("employer_references_rejection_reason"),
    _("employer_references_needs_clarification"),
    _("documents_evaluation"),
    _("documents_flag"),
    _("documents_rejection_reason"),
    _("finance_evaluation"),
    _("finance_flag"),
    _("finance_rejection_reason"),
    _("validity_evaluation"),
    _("validity_flag"),
    _("validity_rejection_reason"),
    _("occupancy_evaluation_prio_2"),
    _("occupancy_evaluation_flag"),
    _("occupancy_evaluation_rejection_reason"),
    _("occupancy_evaluation_needs_clarification"),
    _("occupancy_evaluation_prio_3"),
    _("pets_evaluation"),
    _("pets_flag"),
    _("pets_rejection_reason"),
    _("solvency_evaluation_prio_1"),
    _("solvency_evaluation_prio_2"),
    _("solvency_evaluation_prio_3"),
    _("solvency_evaluation_flag"),
    _("solvency_evaluation_rejection_reason"),
    _("solvency_evaluation_needs_clarification"),
    _("mobility_evaluation"),
    _("mobility_flag"),
    _("mobility_rejection_reason"),
    _("experience_evaluation"),
    _("experience_flag"),
    _("experience_rejection_reason"),
    _("commitment_evaluation"),
    _("commitment_flag"),
    _("commitment_rejection_reason"),
    _("urgency_evaluation"),
    _("urgency_flag"),
    _("urgency_rejection_reason"),
    _("urgency_needs_clarification"),
    _("presentation_evaluation"),
    _("presentation_flag"),
    _("presentation_rejection_reason"),
    _("presentation_needs_clarification"),
    _("priority_evaluation"),
    _("priority_flag"),
    _("priority_rejection_reason"),
    _("priority_needs_clarification"),
    _("motivation_evaluation"),
    _("applicant_comment_evaluation"),
    _("applicant_comment_flag"),
    _("applicant_comment_rejection_reason"),
    _("applicant_comment_needs_clarification"),
    _("singleroom_evaluation"),
    _("singleroom_flag"),
    _("singleroom_rejection_reason"),
    _("singleroom_needs_clarification"),
    _("craftroom_evaluation"),
    _("craftroom_flag"),
    _("craftroom_rejection_reason"),
    _("craftroom_needs_clarification"),
    _("studio_evaluation"),
    _("studio_flag"),
    _("studio_rejection_reason"),
    _("studio_needs_clarification"),
    _("subsidy_evaluation"),
    _("subsidy_flag"),
    _("subsidy_rejection_reason"),
    _("subsidy_needs_clarification"),
    _("overall_comment_second_evaluator"),
    _("landlord_references_comment"),
    _("employer_references_comment"),
    _("occupancy_evaluation_comment"),
    _("solvency_evaluation_comment"),
    _("mobility_comment"),
    _("urgency_comment"),
    _("priority_comment"),
    _("motivation_flag"),
    _("motivation_rejection_reason"),
    _("motivation_needs_clarification"),
    _("motivation_comment"),
    _("applicant_comment_comment"),
    _("singleroom_comment"),
    _("craftroom_comment"),
    _("studio_comment"),
    _("subsidy_comment"),
    _("subsidy_needs_clarification"),
    _("no_pub"),
    _("active.signed"),
    _("active.open"),
    _("active.sent"),
    _("with_offers"),
    _("car_reason"),
    _("move_in_date"),
    _("source_web"),
    _("wants_mobility"),
    _("wants_stockroom"),
    _("family_apartment"),
    _("wants_mobility"),
    _("agreement_deposit"),
    _("apartment_visited"),
    _("car_documentation"),
    _("music_instruments"),
    _("addroom_intended_use"),
    _("number_cycle_trailer"),
    _("addroom_intended_use"),
    _("music_instruments_type"),
    _("stockroom_intended_use"),
    _("subvention_residence_since"),
    _("wants_addroom"),
    _("moving_date"),
    _("with_feedback"),
    _("without_feedback"),
    _("create_task"),
    _("adults_only_name"),
    _("children_only_name"),
    _("virtual"),
    _("pub_start"),
    _("pub_end"),
    _("title_without_link"),
    _("building_adress"),
    _("overall_evaluation_marks"),
    _("evaluation_state"),
    _("evaluation_marks_each"),
    _("evaluation_favorite"),
    _("direct_assigment"),
    _("adults"),
    _("active_prio_title_temp"),
    _("active_prio_listing"),
    _("prios_with_property_type"),
    _("enforcement_monthly_rate"),
    _("landlord_ref"),
    _("permit_doc"),
    _("pub start"),
    _("pub end"),
    _("title without link"),
    _("building adress"),
    _("overall evaluation marks"),
    _("evaluation state"),
    _("evaluation marks each"),
    _("evaluation favorite"),
    _("direct_assigment"),
    _("active prio title temp"),
    _("active prio listing"),
    _("prios with property type"),
    _("enforcement monthly rate"),
    _("landlord ref"),
    _("permit doc"),
    _("adults only name"),
    _("children only name"),
    _("overall evaluation"),
    _("create task"),
    _("other_succesions"),
    _("collectable_interests"),
    _("start_date"),
    _("application"),
    _("assignment_date"),
    _("answer"),
    _("voucher"),
    _("application"),
    _("comments"),
    _("occupation_detail"),
    _("place_of_citizenship"),
    _("has_acceptance"),
    _("has_offer"),
    _("number_of_no_response"),
    _("random_seleted"),
    _("state_of_process"),
    _("flag"),
    _("has_offer"),
    _("random_seleted"),
    _("state_of_process"),
    _("ug_status"),
    _("ug_property_type"),
    _("actual"),
    _("HouseholdListView"),
    _("all assigned"),
    _("assigment_other"),
    _("assigment_parking"),
    _("all"),
    _("with_r"),
    _("without_r"),
    _("hh_r"),
    _("form_id"),
    _("min_max"),
    _("saved_data"),
    _("second_form_id"),
    _("team"),
    _("temp.first_application"),
    _("temp.first_application_inspection"),
    _("temp.first_application_inspection_present"),
    _("was there"),
    _("was_there"),
    _("sum_bicycles_dypoints"),
    _("prio_1_min_income_fulfilled_dypoints"),
    _("prio_1_max_income_fulfilled_dypoints"),
    _("num_children_dypoints"),
    _("num_children"),
    _("num_adults_dypoints"),
    _("num_adults"),
    _("household_income_dypoints"),
    _("household_income"),
    _("state_dypoints"),
    _("incidentals"),
    _("Parking_add"),
    _("randomlog"),
    _("num_inspection"),
    _("prio_1_household_type_fulfilled_dypoints"),
    _("prio_1_age_structure_fulfilled_dypoints"),
    _("EvaluationCraftroomForm"),
    _("EvaluationSingleroomForm"),
    _("EvaluationStudioForm"),
    _("total_points"),
    _("Einfach"),
    _("einfach"),
    _("fehlerhaft"),
    _("versendet"),
    _("landlord_reference_call"),
    _("child"),
    _("adult"),
    _("notify_type"),
    _("notify_date"),
    _("created_by_staff"),
    _("active_prio_apartment_count"),
    _("adult_ages"),
    _("adult_names"),
    _("all_assigned"),
    _("building_title"),
    _("children_ages"),
    _("children_present"),
    _("evaluation_enforcement_doc"),
    _("evaluation_impression"),
    _("evaluation_needs_clarification"),
    _("evaluation_overall"),
    _("evaluation_permit_doc"),
    _("has_free_prios"),
    _("is_assigned"),
    _("list_enforcement_docs"),
    _("names"),
    _("num_men"),
    _("num_refs_employer_ok"),
    _("num_refs_landlord_ok"),
    _("num_women"),
    _("pretty_num_rooms_prio_1"),
    _("pretty_num_rooms_prio_2"),
    _("pretty_num_rooms_prio_3"),
    _("pretty_rent_percentage_prio_1"),
    _("pretty_rent_percentage_prio_2"),
    _("pretty_rent_percentage_prio_3"),
    _("pretty_rent_prio_1"),
    _("pretty_rent_prio_2"),
    _("pretty_rent_prio_3"),
    _("pretty_visited"),
    _("prio_1_age_structure_fulfilled"),
    _("prio_1_household_type_fulfilled"),
    _("prio_1_max_fortune_fulfilled"),
    _("prio_1_max_income_fulfilled"),
    _("prio_1_min_fortune_fulfilled"),
    _("prio_1_min_income_fulfilled"),
    _("prio_ids"),
    _("type"),
    _("wealth_net"),
    _("age"),
    _("fullname"),
    _("fullname_address"),
    _("fullname_birthday"),
    _("type"),
    _("fullname_short"),
    _("is_assigned"),
    _("has_enforcement_doc"),
    _("letter_post"),
    _("pretty_flat"),
    _("pretty_formal"),
    _("pretty_other"),
    _("pretty_parking"),
    _("pretty_title"),
    _("sex"),
    _("type"),
    _("valid_evaluation"),
    _("application"),
    _("prio_1_property_type"),
    _("Year of construction"),
    _("Min max rooms"),
    _("Min max miete"),
    _("craft"),
    _("Craft"),
    _("Allow edit"),
    _("Allow delete"),
    _("Show reloaction"),
    _("Show objects"),
    _("Show waitlist"),
    _("employee_contract"),
    _("employee_reception"),
    _("employee_viewing"),
    _("admin"),
    _("customer"),
    _("Object Representation"),
    _("timestamp"),
    _("actor"),
    _("landlord_reference"),
    _("landlord_reference_reminder"),
    _("employer_reference"),
    _("employer_reference_reminder"),
    _("denied_rent"),
    _("offer"),
    _("offer_alternative"),
    _("colony"),
    _("adress"),
    _("City"),
    _("filter location"),
    _("filter rooms"),
    _("filter rent"),
    _("filter elevator"),
    _("filter floor"),
    _("filter property type"),
    _("app_ang"),
    _("app_act"),
    _("app_ver"),
    _("app_ohn"),
    _("new_evaluated"),
    _("new_application"),
    _("application_state_donut"),
    _("vacant_apartment"),
    _("applications_by_household"),
    _("vega_test"),
    _("origin_map"),
    _("table_when"),
    _("age_income"),
    _("age_pyramid"),
    _("household_radar"),
    _("rent_roi"),
    _("rental_divider"),
    _("publicity_divider"),
    _("demography_divider"),
    _("finance_divider"),
    _("age_gender_violin"),
    _("nation_bfs_bubble"),
    _("commitment_with_contract"),
    _("commitment_without_contract"),
    _("declined_today"),
    _("in_progress"),
    _("with_offer"),
    _("reason_customer"),
    _("canton_map"),
    _("clock"),
    _("nation_bubble"),
    _("nation_bubble_bfs"),
    _("ht_be_bo"),
    _("offer_numbers"),
    _("household_bar"),
    _("min_max_rooms"),
    _("min_max_miete"),
    _("price_sensivity_by_prio"),
    _("sunburst_test"),
    _("open_todos"),
    _("mail_numbers"),
    _("income_rent_heat"),
    _("process_steps"),
    _("age_be_bo"),
    _("nat_be_bo"),
    _("age_class_state_heat"),
    _("age_10_0"),
    _("age_0_4"),
    _("age_5_9"),
    _("age_10_14"),
    _("age_15_19"),
    _("age_20_29"),
    _("age_30_39"),
    _("age_40_49"),
    _("age_50_64"),
    _("age_65_79"),
    _("age_80_120"),
    _("parkingspace_numbers"),
    _("offer_user_numbers"),
    _("first_name"),
    _("username"),
    _("last_name"),
    _("role_text"),
    _("confirmation"),
    _("is_active"),
    _("role"),
    _("teams"),
    _("co_businessplan_doc"),
    _("co_certificate_of_registration"),
    _("co_enforcement_doc"),
    _("co_bylaws_doc"),
    _("reason"),
    _("basement_area"),
    _("Usage"),
    _("employer_reference_call"),
    _("has_no_waitlist_offer_for"),
    _("total_points_swm"),
    _("wealth_gross"),
    _("documents_saved"),
    _("good_salary"),
    _("offer_write"),
    _("objectdata_read"),
    _("objectdata_write"),
    _("get_references"),
    _("add_application"),
    _("export_data"),
    _("change_state"),
    _("send_mails"),
    _("execute_random"),
    _("import_data"),
    _("config_view"),
    _("cost_prov"),
    _("cost_def"),
    _("cost_pred"),
    _("household_income"),
    _("same_adress"),
    _("initial_training"),
    _("initial_training_until"),
    _("co_name"),
    _("co_vat"),
    _("co_vat_number"),
    _("landlordmail"),
    _("employermail"),
    _("employment_contract_terminated"),
    _("employment_contract_terminated_detail"),
    _("send_offer_date"),
    _("tenant_reaction_date"),
    _("valid_till"),
    _("assigment_created"),
    _("pdf_floor_plan"),
    _("object_info"),
    _("entry_date"),
    _("application_info"),
    _("mail_problems"),
    _("acceptance_today"),
    _("measure_category_stacked"),
    _("count_apa_state_icons"),
    _("internet_source"),
    _("source"),
    _("count_apa_state_parkingspace"),
    _("count_apa_state_other"),
    _("apartment_state_donut"),
    _("letting_state_by_date"),
    _("apartment_state_by_room"),
    _("denial_by_reason"),
    _("Checkbox select multiple"),
    _("Float"),
    _("Content text"),
    _("honorary_percent"),
    _("last_edited"),
    _("info"),
    _("employer_reference_date"),
    _("email_control"),
    _("landlord_reference_date"),
    _("employer_reference_number"),
    _("landlord_reference_number"),
    _("has_offer_reason"),
    _("side_info_communication"),
    _("parking_type"),
    _("doc_notice_of_termination"),
    _("prio_1_max_fortune_fulfilled_dypoints"),
    _("prio_1_min_fortune_fulfilled_dypoints"),
    _("child_doc"),
    _("relocation_reason_detail"),
    _("relocation_reason_occupancy"),
    _("relocation_reason_rent"),
    _("relocation_reason_termination"),
    _("simple"),
    _("Simple"),
    _("already_tenant"),
    _("household_income"),
    _("occupancy_denseness"),
    _("rent_burden"),
    _("has_offer_link"),
    _("already_tenant"),
    _("notice_of_termination_detail"),
    _("current_rooms"),
    _("current_rent"),
    _("sightseeing_schedule"),
    _("apply_button"),
    _("plz"),
    _("building_title_multiselect"),
    _("prio_1_rooms"),
    _("date_of_assigment"),
    _("wants_parking"),
    _("car_doc"),
    _("subvention_doc"),
    _("employment_contract"),
    _("offer_send_mail"),
    _("movie_in_date_back"),
    _("pretty_rent_percentage_prio_1_dypoints"),
    _("notice_of_termination_doc"),
    _("bfu_regions_multiselect"),
    _("bfu_regions"),
    _("contract"),
    _("side_info_points"),
    _("metropolitan"),
    _("Building"),
    _("Usage"),
    _("joint_tax_return"),
    _("certificate_of_unpaid_debts"),
    _("obstacle_free"),
    _("applicant_list"),
    _("direct_select_output"),
    _("direct_select"),
    _("direct_select_with_comment"),
    _("side_info_points_swm"),
    _("random_select"),
    _("rent_interest_charge"),
    _("Property type"),
    _("publicated_adress"),
    _("create_object_application"),
    _("academic_title"),
    _("additional_doc"),
    _("assigment_application_date"),
    _("assigment_start_date"),
    _("honorary"),
    _("honorary_charging"),
    _("mv_assigment"),
    _("mv_sent"),
    _("mv_signed"),
    _("parking_assigment"),
    _("tenant_names"),
    _("year_rentalgross_net"),
    _("year_rentalgross_net_is"),
    _("year_rentalgross_net_square"),
    _("co_area"),
    _("location"),
    _("craft_type"),
    _("time_apartment_search"),
    _("personal_liability_insurance"),
    _("household_insurance"),
    _("wants_rooftop"),
    _("rooftop"),
    _("securities_details_name"),
    _("television_provider"),
    _("television_provider_detail"),
    _("attachments"),
    _("citizen"),
    _("user_feedback"),
    _("car_license_plate_number"),
    _("intended_use"),
    _("adult_id"),
    _("bounce"),
    _("houselhold_id"),
    _("letter"),
    _("message_id"),
    _("msg"),
    _("notify"),
    _("household_id"),
    _("application_select"),
    _("log_select"),
    _("view_select"),
    _("relocation_select"),
    _("message_de"),
    _("signature_de"),
    _("subject_de"),
    _("number_adults"),
    _("number_children"),
    _("swm_doc"),
    _("swm_sub_doc"),
    _("number_of_declining_tenant"),
    _("number_of_declining_landlord"),
    _("site_title"),
    _("project_name"),
    _("contact_tel_number"),
    _("contact_mail_adress"),
    _("contact_address"),
    _("website"),
    _("notyet"),
    _("landlord"),
    _("tenant"),
    _("wants_other_listing"),
    _("hobbyroom"),
    _("stockroom"),
    _("workshop"),
    _("assignment_craft"),
    _("rooftop"),
    _("new_tasks_half"),
    _("application_today_half"),
    _("lost_mails_half"),
    _("end_date"),
    _("needs_discussion"),
    _("comment"),
    _("accounting_cleared"),
    _("accounting_date"),
    _("area_rent"),
    _("direct_offer"),
    _("advertisement_end"),
    _("send_mail_craft"),
    _("send_mail_apartment"),
    _("send_mail_other"),
    _("send_mail_virtual"),
    _("send_mail_simple"),
    _("send_mail_parkingspace"),
    _("send_mail_inspection"),
    _("Virtual"),
    _("co_legal_form"),
    _("co_industry"),
    _("co_founding_year"),
    _("isometry"),
    _("situation_plan"),
    _("evaluator_first"),
    _("evaluator_second_evaluator"),
    _("direct_alternatives"),
    _("info_checklist"),
    _("adv_move_in_date"),
    _("rank"),
    _("direct_offer"),
    _("advertisement_end"),
    _("num_acceptance"),
    _("num_applications_prio_1"),
    _("num_rented"),
    _("mittel"),
    _("co_legal_form"),
    _("co_industry"),
    _("co_certificate_of_registration_doc"),
    _("disability_pension"),
    _("prio_1_object"),
    _("deduction_of_tax_at_source"),
    _("income_gross_net"),
    _("child_custody"),
    _("log_rank"),
    _("cp_to_tenant"),
    _("pin_rank"),
    _("relocation_request"),
    _("relocation_request_craft"),
    _("relocation_request_other"),
    _("relocation_request_parking"),
    _("wait_list"),
    _("household_income_input"),
    _("contract_state"),
    _("object_repr"),
    _("loggia"),
    _("swm_recalc"),
    _("reference_number"),
    _("target_group"),
    _("deposit"),
    _("denied_reason"),
    _("car_waiver"),
    _("has_car"),
    _("has_motorcycle"),
    _("has_parking"),
    _("has_parking_detail"),
    _("income_group"),
    _("edit_model"),
    _("delete_model"),
    _("references_landlord_evaluation"),
    _("references_employer_evaluation"),
    _("contact"),
    _("out_rooms"),
    _("out_object_type"),
    _("out_area"),
    _("out_rent"),
    _("co_legal_form"),
    _("co_name"),
    _("co_industry"),
    _("co_certificate_of_registration"),
    _("co_vat"),
    _("co_businessplan_doc"),
    _("present_in_apartment"),
    _("offer_send_mail_user"),
    _("send_waitlist"),
    _("send_mail_parkinspace"),
    _("new_task"),
    _("insurance_agent_template"),
    _("wants_stockroom_detail"),
    _("wants_hobbyroom_detail"),
    _("num_ecars_special"),
    _("lang"),
    _("household_type_detail"),
    _("household_income_net"),
    _("apartment_visited_detail"),
    _("wohnungsnummern"),
    _("zusammenlegen"),
    _("constructional_adaptations_detail"),
    _("freezer"),
    _("engagement_filter"),
    _("constructional_adaptations"),
    _("household_income_net"),
    _("career_change"),
    _("career_change_detail"),
    _("working_since_in_years"),
    _("enforcement_detail "),
    _("commuter_type"),
    _("child_of_parent"),
    _("tender_title"),
    _("loggia_area"),
    _("garden_sitting_place_area"),
    _("rentalprice_squaremeter_net"),
    _("rentalprice_squaremeter"),
    _("terrace_area"),
    _("notice_of_termination_date"),
    _("employment_fixedterm_date"),
    _("employment_fixedterm"),
    _("employment_contract_terminated_date"),
    _("phone_mobile"),
    _("Evaluate marked objects as tenants"),
    _("prerenter"),
    _("curator"),
    _("relocationrequests"),
    _("previous_rooms"),
    _("employment_type"),
    _("company_division"),
    _("temporary_employment"),
    _("company_organisational_unit"),
    _("child_employed"),
    _("Commitment evaluation"),
    _("Mobility evaluation"),
    _("Priority evaluation"),
    _("Acomment evaluation"),
    _("Presentation evaluation"),
    _("Status Application"),
    _("Household evaluation"),
    _("Instrument evaluation"),
    _("Document evaluation"),
    _("Solvency evaluation"),
    _("Urgency evaluation"),
    _("Pets evaluation"),
    _("Urgency evaluation"),
    _("Validity evaluation"),
    _("Document evaluation"),
    _("income_rent_ratio"),
    _("Parking detail"),
    _("parking detail"),
    _("bfs_region"),
    _("without_offers"),
    _("project_reference"),
    _("project_reference_detail"),
    _("project_reference_duration"),
    _("project_reference_since"),
    _("munich_doc"),
    _("social_housing_doc"),
    _("temporary_tenant"),
    _("unobstructed_doc"),
    _("notvacant"),
    _("min_max_rooms_0"),
    _("min_max_rooms_1"),
    _("min_max_miete_0"),
    _("min_max_miete_1"),
    _("min_max_rent_0"),
    _("min_max_rent_1"),
    _("adults_exist"),
    _("children_exist"),
    _("children_gender"),
    _("date_of_event"),
    _("disability_certificate_doc"),
    _("priorities_input"),
    _("social_points"),
    _("urgency_points"),
    _("social_people"),
    _("social_number"),
    _("social_rooms"),
    _("social_validity"),
    _("layout_plan"),
    _("send_mail_to_curator"),
    _("contract_details"),
    _("signed_on"),
    _("year_of_construction"),
    _("relocation_apartment_comment"),
    _("garden"),
    _("disability_pension_quota"),
    _("company_until"),
    _("additional_costs_1"),
    _("additional_costs_2"),
    _("assignment_history"),
    _("erp_apartment"),
    _("incidentals"),
    _("disability_pension_quota"),
    _("disability_pension_quota"),
    _("disability_pension_quota"),
    _("wait_list_min_max_miete"),
    _("wait_list_min_max_rooms"),
    _("paused"),
    _("relocation_reasons"),
    _("proprietor"),
    _("inactive.anonym"),
    _("normal"),
    _("subvention"),
    _("social"),
    _("insurance"),
    _("deposit"),
    _("situation_plan"),
    _("factsheets"),
    _("domicile_same_municipality"),
    _("domicile_same_municipality_detail"),
    _("work_same_municipality"),
    _("work_same_municipality_detail"),
    _("domicile_same_municipality_past"),
    _("domicile_same_municipality_past_detail"),
    _("relatives_same_municipality"),
    _("relatives_same_municipality_detail"),
    _("adult_is_member"),
    _("application_pdf"),
    _("building_publicated_adress"),
    _("building_street"),
    _("factsheets"),
    _("object_select"),
    _("fake_field_for_show_error"),
    _("file"),
    _("object_select_1"),
    _("object_select_2"),
    _("previous_offer_declined"),
    _("securities_detail"),
    _("securities_details"),
    _("timeslots"),
    _("adult_is_member"),
    _("adult_is_member"),
    _("adult_is_member"),
    _("number_of_declining_landlord_old"),
    _("employment_contract_doc"),
    _("passport_doc"),
    _("company_name"),
    _("company_founding_year"),
    _("company_annual_turnover"),
    _("company_contact"),
    _("company_contact_phone"),
    _("company_contact_email"),
    _("current_housing_situation"),
    _("landlord_contact"),
    _("landlord_contact_phone"),
    _("landlord_contact_email"),
    _("current_housing_situation_detail"),
    _("move_three_years"),
    _("previous_street_nr"),
    _("previous_postcode"),
    _("previous_city"),
    _("has_credittrust"),
    _("credittrust_upload"),
    _("income_doc"),
    _("additional_doc_2"),
    _("additional_doc_3"),
    _("agreement_truth"),
    _("parking_regular"),
    _("pets_type"),
    _("securities_provider"),
    _("securities_provider_details"),
    _("source_detail"),
    _("agreement_penalty"),
]


FORM_FIELD_CUSTOM_TRANSLATION = [
    _("bfs_region"),
    _("employer_reference_finished"),
    _("swm"),
    _("income_net"),
    _("income_net_additional"),
    _("income_gross_additional"),
    _("confirmation_information"),
    _("previous_area"),
    _("agreement_data"),
    _("income_rent_ratio"),
    _("parking_detail"),
    _("volunteering"),
    _("volunteering_hours"),
]


def is_snake_case(string):
    # Check if the string contains only lowercase letters, digits, and underscores
    if re.match(r"^[a-z0-9_]+$", string):
        # Check if the string contains an underscore
        if "_" in string:
            return True
    return False


class Command(BaseCommand):
    def handle(self, *args, **options):
        """Handle command."""
        log.info("Start!")
        for key in FORM_FIELD_CUSTOM_TRANSLATION + CUSTOM_TRANSLATION:
            key = str(key)

            try:
                tk = TranslationKey.objects.get(id_name=key)
            except TranslationKey.DoesNotExist:
                log.error(f"Key not found: {key}")
                continue

            if is_snake_case(key):
                tk.snake_name = key

            if not tk.views:
                tk.views = ["custom"]
            elif "custom" not in tk.views:
                tk.views.append("custom")

            try:
                tk.save()
                log.info(f"Updated: {key}")
            except django.db.utils.IntegrityError:
                log.error(f"Duplicate key: {key}")
                continue

        log.info("Done!")



from django.urls import path

from translate.service.views import (
    LivelinessCheckView,
    ReadinessCheckView,
    TranslationsAPIView,
    TranslationsAPIViewV2,
)

urlpatterns = [
    path("v1/translations/<language>/", TranslationsAPIView.as_view(), name="api_translations"),
    path("v2/translations/<language>/", TranslationsAPIViewV2.as_view(), name="api_translations_v2"),
    path("status/liveliness", LivelinessCheckView.as_view(), name="status_liveliness"),
    path("status/readiness", ReadinessCheckView.as_view(), name="status_readiness"),
]



from pathlib import Path

file_name = "VERSION"

root = Path(__file__).resolve().parent.parent.parent.parent

path_to_version = root.joinpath(file_name)


def extract_version():
    content_dict = {}
    content_list = Path(path_to_version).read_text().splitlines()

    for item in content_list:
        # NOTE: Ignore comments within the version files
        if item.startswith("#") or item.startswith("//"):
            continue

        # NOTE: Ignore all value components (ie. value contains `=`)
        key, *tail = item.split("=")

        content_dict[key] = "".join(tail)

    tag = content_dict.get("VERSION")

    if not tag:
        raise ValueError("project version not set - add `VERSION` variable")

    return tag



import glob
import json
import operator
from collections import defaultdict
from pathlib import Path
from typing import Any, Dict

import django.db.utils
import polib
from django.conf import settings
from django.conf.locale import LANG_INFO
from django.core.management import call_command
from django.core.management.base import BaseCommand

from translate.core.utils.logger import log
from translate.service.models import Language, Translation, TranslationKey

SNAKE_TRANSLATIONS = Path("snake_translations.json")
# DEFAULT_DIR_PATH = Path("translate/service/tests/fixtures")  # This is a testing directory
DEFAULT_DIR_PATH = Path("export_translations")


def extract_language(path: str) -> str:
    """Extract language from path with locale."""
    parts = path.split("/")
    idx = parts.index("locale")
    return parts[idx + 1]


class Command(BaseCommand):
    help = "Importing translation keys command from .json and .po files"

    def add_arguments(self, parser):
        """Attach argument for import_translations command."""
        parser.add_argument(
            "translations_dir",
            type=str,
            nargs="?",
            default=DEFAULT_DIR_PATH,
        )

    @staticmethod
    def read_po_files(dirs) -> dict:
        """Read po files."""
        log.info("Reading po files.")
        po_files = {
            po_file: polib.pofile(po_file, encoding="utf-8")
            for po_file in glob.iglob(f"{dirs}/**/*.po", recursive=True)
        }

        files_with_languages = defaultdict(list)
        for file_path, file in po_files.items():
            lang = extract_language(file_path)
            files_with_languages[lang].append((file_path, file))

        log.info("Done")
        return files_with_languages

    @staticmethod
    def create_po_keys(file_entries):
        """Create po keys."""

        existing_po_keys = {key.id_name: key for key in TranslationKey.objects.all()}

        new_po_keys = {
            entry.msgid: TranslationKey(
                id_name=entry.msgid,
                id_name_plural=entry.msgid_plural,
                encoding=entry.encoding,
                usage_context=entry.msgctxt,
                occurrences=sorted(map(operator.itemgetter(0), entry.occurrences)),
                flags=entry.flags,
            )
            for entry in file_entries
            if entry.msgid not in existing_po_keys.keys()
        }

        return existing_po_keys, new_po_keys

    @staticmethod
    def create_po_translations(file_entries, key_entries, language) -> list:
        """Create po translations."""

        entries = [
            Translation(
                language=language,
                key=key_entries.get(entry.msgid),
                translation=entry.msgstr if entry.msgstr else entry.msgid,
                translation_plural=entry.msgid_plural,
            )
            for entry in file_entries
        ]

        return entries

    @staticmethod
    def process_po_files(dirs):
        """Process po files."""
        po_files = Command.read_po_files(dirs)

        existing_keys, new_key_entries = {}, {}
        for lang, files in sorted(po_files.items()):
            for file_path, file_entries in files:
                d1, d2 = Command.create_po_keys(file_entries)
                existing_keys = {**existing_keys, **d1}
                new_key_entries = {**new_key_entries, **d2}

        log.info(f"Found {len(existing_keys.items())} already existing TranslationKeys.")
        log.info(f"Prepared {len(new_key_entries.items())} of new TranslationKeys via po files.")

        inserts = TranslationKey.objects.bulk_create(new_key_entries.values(), ignore_conflicts=True)
        if not inserts:
            log.info("NO NEW PO TRANSLATIONS HAVE BEEN ADDED")
        else:
            log.info(f"Done. Inserted {len(inserts)} TranslationKeys from po files.")
        log.info("----------------------------------------------------")

        all_keys = {**existing_keys, **new_key_entries}

        translations = defaultdict(list)
        for lang, files in sorted(po_files.items()):
            language, _ = Language.objects.get_or_create(lang_info=lang)
            for file_path, file_entries in files:
                translations[lang].extend(Command.create_po_translations(file_entries, all_keys, language))

        for lang, translations_list in translations.items():
            translation_inserts = []
            integrity_errors = 0
            try:
                log.info(f"Prepared {len(translations_list)} Translations for {lang} language. Inserting bulk... ")
                translation_inserts = Translation.objects.bulk_create(translations_list, ignore_conflicts=True)
            except django.db.utils.IntegrityError:
                log.info("Integrity error occurred, and bulk create was canceled! Inserting individually...")
                for obj in translations_list:
                    try:
                        obj.save()
                        translation_inserts.append(obj)
                    except django.db.utils.IntegrityError:
                        integrity_errors += 1

            log.info(f"Integrity error occurred with {integrity_errors} Translations for {lang} language.")
            if not translation_inserts:
                log.info("NO NEW PO TRANSLATIONS HAVE BEEN ADDED")
            log.info(f"Done. Inserted {len(translation_inserts)} Translations from po files for {lang} language.")
            log.info("----------------------------------------------------")

    @staticmethod
    def create_json_keys(language_data, language_code):
        """Create language data."""
        log.info("Preparing translation keys.")
        keys = [
            TranslationKey(
                snake_name=snake_name, id_name=obj.get("translations"), views=[source_dict] + obj.get("source")
            )
            for source_dict, value in language_data.items()
            for snake_name, obj in value.items()
        ]
        key_dict = {}
        for key in keys:
            key_dict[key.snake_name] = key

        log.info(f"Prepared {len(keys)} RAW TranslationKeys for {language_code} language.")

        log.info("Cleaning")
        already_imported_sn = Command.json_keys_sanity_check(keys)
        for name in already_imported_sn:
            if name in key_dict.keys():
                key_dict.pop(name)

        bulk_inserts = key_dict.values()
        if not bulk_inserts:
            log.info("NO NEW KEYS HAVE BEEN INSERTED!")
        else:
            log.info("Inserting in bulk...")
            objs = TranslationKey.objects.bulk_create(bulk_inserts, ignore_conflicts=True)
            log.info("Done!")
            log.info(f"Imported {len(objs)} TranslationKeys into database!")

        return key_dict

    @staticmethod
    def json_keys_sanity_check(obj_keys) -> list:
        """
        This method returns a list of already imported key objects snake_names
        """
        snake_names = []
        for key in obj_keys:
            snake_names.append(key.snake_name)

        already_existing_keys = TranslationKey.objects.in_bulk(snake_names, field_name="snake_name")
        log.info(f"Already existing json keys and snake names {len(already_existing_keys.keys())} founded in database.")
        return already_existing_keys.keys()

    @staticmethod
    def read_json_translations(dirs):
        """Read JSON translations."""
        log.info("Reading json translations")
        translations = json.loads((settings.BASE_DIR / dirs / SNAKE_TRANSLATIONS).read_text())
        log.info("Done")
        log.info(f"There are {len(translations)} available languages in this file.")
        return translations

    @staticmethod
    def create_json_translations(translations: Dict[str, Any], lang_key: str):
        """Import snake_names for a specific language."""
        log.info(f"Processing snake_names from JSON for {lang_key} language.")
        language = Language.objects.get(lang_info=lang_key)
        data_for_provided_language = translations.get(lang_key)

        bulk_translations = []
        for _, dict_data in data_for_provided_language.items():
            snake_names_list = list(dict_data.keys())
            keys = {key.snake_name: key for key in TranslationKey.objects.filter(snake_name__in=snake_names_list)}

            for snake_key, snake_data in dict_data.items():
                translation_key = keys.get(snake_key)
                if translation_key:
                    translation = Translation(
                        language=language,
                        key=translation_key,
                        translation=snake_data.get("translations"),
                        translation_plural=snake_data.get("translations"),
                    )
                    bulk_translations.append(translation)

        try:
            Translation.objects.bulk_create(bulk_translations, ignore_conflicts=True)
        except django.db.utils.IntegrityError:
            log.info("There was a Integrity Error with json files, none were inserted.")
        log.info("Done")
        log.info("----------------------------------------------------")

    @staticmethod
    def process_json_files(dirs):
        """Process json files."""
        translations = Command.read_json_translations(dirs)

        _ = {
            lang: Language.objects.get_or_create(lang_info=lang)
            for lang in translations
            if lang in list(LANG_INFO.keys())
        }

        for lang in translations:
            _ = Command.create_json_keys(translations.get(lang), lang)
            _ = Command.create_json_translations(translations, lang)

    def handle(self, *args, **options):
        """Entrypoint to the command."""

        dir_name = Path(settings.BASE_DIR, options.get("translations_dir"))

        Command.process_json_files(dir_name)
        Command.process_po_files(dir_name)

        call_command("cleanup_custom_translation")



import pytest


class TestFilters:
    @pytest.mark.filter
    @pytest.mark.django_db
    def test_translation_melon_filter(self, translation_factory):
        """
        Test for filtering Translations that have keys with valid by convention snake_names, not randomly generated ones
        """
        from translate.service import admin
        from translate.service.filters import (
            FILTER_TRANSLATIONS_MELON,
            IgnoreMimoTranslationsFilter,
        )
        from translate.service.models import Translation

        obj, _, _ = translation_factory

        parameter_name = IgnoreMimoTranslationsFilter.parameter_name

        translation_filter_melon = IgnoreMimoTranslationsFilter(
            None,
            {parameter_name: FILTER_TRANSLATIONS_MELON},
            Translation,
            admin.TranslationAdmin,
        )

        melon_obj_qs = translation_filter_melon.queryset(None, None)
        values_list = melon_obj_qs.values_list("id", flat=True)

        assert melon_obj_qs
        assert obj.id in values_list

    @pytest.mark.filter
    @pytest.mark.django_db
    def test_translation_mimo_filter(self, translation_mimo_factory):
        """
        Test for filtering Translations that are connected with keys that have randomly generated snake_name_UUID
        """
        from translate.service import admin
        from translate.service.filters import (
            FILTER_TRANSLATIONS_MIMO,
            IgnoreMimoTranslationsFilter,
        )
        from translate.service.models import Translation

        obj, _, _ = translation_mimo_factory
        parameter_name = IgnoreMimoTranslationsFilter.parameter_name

        translation_filter_mimo = IgnoreMimoTranslationsFilter(
            None,
            {parameter_name: FILTER_TRANSLATIONS_MIMO},
            Translation,
            admin.TranslationAdmin,
        )

        mimo_obj_qs = translation_filter_mimo.queryset(None, None)
        values_list = mimo_obj_qs.values_list("id", flat=True)

        assert mimo_obj_qs
        assert obj.id in values_list

    @pytest.mark.filter
    @pytest.mark.django_db
    def test_translation_key_filter(self, translation_key_factory):
        """
        Test for filtering TranslationKeys that have keys with snake_names valid by convention i.e. melon keys
        """
        from translate.service import admin
        from translate.service.filters import FILTER_KEYS_MELON, IgnoreMimoKeysFilter
        from translate.service.models import TranslationKey

        translation_key = translation_key_factory
        parameter_name = IgnoreMimoKeysFilter.parameter_name

        translation_key_melon_filter = IgnoreMimoKeysFilter(
            None, {parameter_name: FILTER_KEYS_MELON}, TranslationKey, admin.TranslationKeyAdmin
        )

        melon_obj_qs = translation_key_melon_filter.queryset(None, None)
        values_list = melon_obj_qs.values_list("id", flat=True)
        assert melon_obj_qs
        assert translation_key.id in values_list

    @pytest.mark.filter
    @pytest.mark.django_db
    def test_translation_key_mimo_filter(self, translation_key_mimo_factory):
        """
        Test for filtering TranslationKeys that are connected with legacy keys that have randomly generated snake_names
        """
        from translate.service import admin
        from translate.service.filters import FILTER_KEYS_MIMO, IgnoreMimoKeysFilter
        from translate.service.models import TranslationKey

        translation_key = translation_key_mimo_factory
        parameter_name = IgnoreMimoKeysFilter.parameter_name

        translation_key_mimo_filter = IgnoreMimoKeysFilter(
            None,
            {parameter_name: FILTER_KEYS_MIMO},
            TranslationKey,
            admin.TranslationKeyAdmin,
        )

        mimo_obj_qs = translation_key_mimo_filter.queryset(None, None)
        values_list = mimo_obj_qs.values_list("id", flat=True)

        assert mimo_obj_qs
        assert translation_key.id in values_list

    @pytest.mark.filter
    @pytest.mark.django_db
    def test_context_input_filter(self, translation_key_with_context_factory):
        """
        Test for filtering TranslationKeys based on usage context input
        """
        from translate.service import admin
        from translate.service.filters import (
            FILTER_CONTEXT,
            FILTER_NO_CONTEXT,
            ContextInputFilter,
        )
        from translate.service.models import TranslationKey

        tk_list = translation_key_with_context_factory
        parameter_name = ContextInputFilter.parameter_name

        # NOTE: Since context field is recently added, and its empty across db,
        # we bypassed getting keys with context with new factory

        context_filter = ContextInputFilter(
            None,
            {parameter_name: FILTER_CONTEXT},
            TranslationKey,
            admin.TranslationKeyAdmin,
        )

        context_obj_qs = context_filter.queryset(None, None).filter(id__in=[tk.id for tk in tk_list])
        values_context_list = context_obj_qs.values_list("id", flat=True)

        assert context_obj_qs

        for tk in tk_list:
            assert tk.id in values_context_list

        assert context_obj_qs.count() == len(tk_list), "Unexpected number of keys with context"
        assert set(values_context_list) == set([tk.id for tk in tk_list]), "Unexpected keys with context"

        # Testing filters for keys with no context
        no_context_filter = ContextInputFilter(
            None,
            {parameter_name: FILTER_NO_CONTEXT},
            TranslationKey,
            admin.TranslationKeyAdmin,
        )

        no_context_obj = no_context_filter.queryset(None, None)

        # Assert that there are no TranslationKeys with context
        assert no_context_obj.filter(usage_context__isnull=False).count() == 0, "Unexpected keys with context"



import os
from pathlib import Path

from django.core.wsgi import get_wsgi_application

from libs.env import load_environ
from translate.core.settings import BASE_DIR

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "translate.core.settings")
dotenv_path = Path(BASE_DIR) / ".env"
load_environ(dotenv_path)

application = get_wsgi_application()



import random

import factory
from django.conf.locale import LANG_INFO

from translate.service import models


class TranslationFactory(factory.django.DjangoModelFactory):
    """Testing factory for ``Translation`` model."""

    class Meta:
        model = models.Translation

    translation = factory.Faker("bs")


class LanguageFactory(factory.django.DjangoModelFactory):
    """Testing factory for ``Language`` model."""

    class Meta:
        model = models.Language

    @factory.lazy_attribute
    def lang_info(self):
        """
        Generate random language info, but different
        from the one already available in the database.
        """
        all_lang_keys = set(LANG_INFO.keys())
        existing_lang_keys = set(models.Language.objects.values_list("lang_info", flat=True))
        available_lang_keys = list(all_lang_keys - existing_lang_keys)

        if not available_lang_keys:
            raise ValueError("No available language keys")

        return random.choice(available_lang_keys)


class TranslationKeyFactory(factory.django.DjangoModelFactory):
    """Testing factory for ``TranslationKey`` model."""

    class Meta:
        model = models.TranslationKey


class FakeLanguageFactory(factory.django.DjangoModelFactory):
    """
    Testing factory for ``Language`` model, but a fake one
    because we don't want to deal with the real languages
    that could already be present in the database.
    """

    class Meta:
        model = models.Language

    @factory.lazy_attribute
    def lang_info(self):
        """
        Generate random language info, but different
        from the one already available in the database.
        """
        import string

        letters = string.ascii_letters
        return "".join(random.choice(letters) for _ in range(4))
